@@ -22,6 +22,7 @@
  *     properly.
  */
 
+#define use_automargins 1
 
 /* 
  *  This is where you define a list of ``fallback'' servers in case the client
@@ -41,7 +42,17 @@
  *  the client will NOT compile and work properly!  Use the default here if
  *  you dont have other servers to use.
  */
-#define DEFAULT_SERVER "localhost irc.efnet.net irc.undernet.org irc.dal.net"
+#define DEFAULT_SERVER "irc.efnet.net irc.undernet.org irc.dal.net"
+
+/*
+ * This is the location of your SSL certificate file.
+ * XXX - This needs to be auto-detected by configure, somehow...
+ * XXX - This is the FreeBSD location. how bogus.
+ */
+/*
+#define DEFAULT_SSL_ROOT_CERTS_LOCATION "/usr/local/share/certs/ca-root-nss.crt"
+*/
+#define DEFAULT_SSL_ROOT_CERTS_LOCATION NULL
 
 /*
  * This is an experimental feature to thwart infinite recursion.  It is not
@@ -281,12 +292,14 @@
 #define DEFAULT_DCC_SLIDING_WINDOW 1
 #define DEFAULT_DCC_STORE_PATH NULL
 #define DEFAULT_DCC_USE_GATEWAY_ADDR 0
+#define DEFAULT_DEFAULT_SCRIPT_ENCODING "ISO-8859-1"
 #define DEFAULT_DISPATCH_UNKNOWN_COMMANDS 0
 #define DEFAULT_DISPLAY 1
 #define DEFAULT_DISPLAY_ANSI 1
 #define DEFAULT_DISPLAY_PC_CHARACTERS 4
 #define DEFAULT_DO_NOTIFY_IMMEDIATELY 1
 #define DEFAULT_EIGHT_BIT_CHARACTERS 1
+#define DEFAULT_FIRST_LINE NULL
 #define DEFAULT_FLOATING_POINT_MATH 0
 #define DEFAULT_FLOATING_POINT_PRECISION 16
 #define DEFAULT_FLOOD_AFTER 3
@@ -348,9 +361,9 @@
 #define DEFAULT_STATUS_CHANOP "@"
 #define DEFAULT_STATUS_CLOCK " %T"
 #define DEFAULT_STATUS_CPU_SAVER " (%L)"
-#define DEFAULT_STATUS_FORMAT "%T [%R] %*%=%@%N%#%S%{1}H%H%B%Q%A%C%+%I%O%M%F%L %D %U %W"
-#define DEFAULT_STATUS_FORMAT1 "%T [%R] %*%=%@%N%#%S%{1}H%H%B%Q%A%C%+%I%O%M%F%L %U "
-#define DEFAULT_STATUS_FORMAT2 "%W %X %Y %Z "
+#define DEFAULT_STATUS_FORMAT REV_TOG_STR "%T [%R] %*%=%@%N%#%S%{1}H%H%B%Q%A%C%+%I%O%M%F%L %D %U %W"
+#define DEFAULT_STATUS_FORMAT1 REV_TOG_STR "%T [%R] %*%=%@%N%#%S%{1}H%H%B%Q%A%C%+%I%O%M%F%L %U "
+#define DEFAULT_STATUS_FORMAT2 REV_TOG_STR "%W %X %Y %Z "
 #define DEFAULT_STATUS_HALFOP "%"
 #define DEFAULT_STATUS_HOLD " Held: "
 #define DEFAULT_STATUS_HOLD_LINES "%B"
@@ -418,9 +431,10 @@
 #define DEFAULT_TAB 1
 #define	DEFAULT_TAB_MAX 0
 #define DEFAULT_TERM_DOES_BRIGHT_BLINK 0
+#define DEFAULT_TMUX_OPTIONS NULL
 #define DEFAULT_UNDERLINE_VIDEO 1
 #define DEFAULT_USER_INFORMATION "EPIC5 -- Into the abyss"
-#define DEFAULT_WORD_BREAK ".,; \t"
+#define DEFAULT_WORD_BREAK " \t"
 #define DEFAULT_WSERV_TYPE "screen"
 #define DEFAULT_XTERM "xterm"
 #define DEFAULT_XTERM_OPTIONS NULL
