@@ -15,11 +15,10 @@
 	char *	get_input 			(void);
 	char *	get_input_prompt 		(void);
 	void	init_input 			(void);
-	void	input_move_cursor 		(int, int);
 	char	input_pause 			(char *);
-	void	set_input 			(const char *);
-	void	set_input_prompt 		(void *);
+	void	set_input_prompt		(void *);
 	void	update_input 			(void *, int);
+	int     cursor_position 		(void *s);
 
 /* keybinding functions */
 	BUILT_IN_KEYBINDING(backward_character);
@@ -50,6 +49,7 @@
 	BUILT_IN_KEYBINDING(insert_altcharset);
 	BUILT_IN_KEYBINDING(insert_blink);
 	BUILT_IN_KEYBINDING(insert_bold);
+	BUILT_IN_KEYBINDING(insert_italic);
 	BUILT_IN_KEYBINDING(insert_reverse);
 	BUILT_IN_KEYBINDING(insert_underline);
 	BUILT_IN_KEYBINDING(parse_text);
@@ -58,6 +58,7 @@
 	BUILT_IN_KEYBINDING(send_line);
 	BUILT_IN_KEYBINDING(toggle_insert_mode);
 	BUILT_IN_KEYBINDING(type_text);
+	BUILT_IN_KEYBINDING(debug_input_line);
 
 /* used by update_input */
 #define NO_UPDATE 0
