@@ -16,25 +16,28 @@ struct 	WindowStru;
 typedef struct  status_line {
         char *		raw;
         char *		format;
-        const char *	(*func[MAX_FUNCTIONS]) (struct WindowStru *, int, int);
-	int		map[MAX_FUNCTIONS];
-	int		key[MAX_FUNCTIONS];
+        const char *	(*func[MAX_FUNCTIONS]) (struct WindowStru *, short, char);
+	short		map[MAX_FUNCTIONS];
+	char		key[MAX_FUNCTIONS];
         int   		count;
         char *		result;
 } Status_line;
 
 typedef struct  status_stuff {          
         Status_line     line[3];
-        int             double_status;
-        char            *special;
+        short           double_status;
+        short           number;
+        char *		special;
 } Status;
 
 extern	Status	main_status;
 
 	char *	convert_sub_format (const char *, char);
-	int	make_status 	(struct WindowStru *, int);	/* Don't call */
-	void	build_status 	(const void *);
+	void	compile_status (struct WindowStru *, struct status_stuff *);
+	int	make_status 	(struct WindowStru *, struct status_stuff *);
+	int	redraw_status	(struct WindowStru *, struct status_stuff *);
+	void	build_status 	(void *);
 	int	permit_status_update	(int);
-	void	rebuild_a_status (struct WindowStru *);		/* Don't call */
+	char *	function_status_oneoff	(char *);
 
 #endif /* _STATUS_H_ */
