@@ -47,6 +47,7 @@ int	ALLOW_C1_CHARS_VAR,
 	DISPATCH_UNKNOWN_COMMANDS_VAR,
 	DISPLAY_VAR,
 	DO_NOTIFY_IMMEDIATELY_VAR,
+	FIRST_LINE_VAR,
 	FLOATING_POINT_MATH_VAR,
 	FLOATING_POINT_PRECISION_VAR,
 	FLOOD_AFTER_VAR,
@@ -58,7 +59,6 @@ int	ALLOW_C1_CHARS_VAR,
 	FLOOD_WARNING_VAR,
 	HIDE_PRIVATE_CHANNELS_VAR,
 	HIGHLIGHT_CHAR_VAR,
-	HIGH_BIT_ESCAPE_VAR,
 	HOLD_SLIDER_VAR,
 	INDENT_VAR,
 	INPUT_INDICATOR_LEFT_VAR,
@@ -108,6 +108,7 @@ int	ALLOW_C1_CHARS_VAR,
 	SHOW_CHANNEL_NAMES_VAR,
 	SHOW_NUMERICS_VAR,
 	SHOW_STATUS_ALL_VAR,
+	SSL_ROOT_CERTS_LOCATION_VAR,
 	STATUS_AWAY_VAR,
 	STATUS_CHANNEL_VAR,
 	STATUS_CHANOP_VAR,
@@ -181,8 +182,7 @@ int	ALLOW_C1_CHARS_VAR,
 	SUPPRESS_FROM_REMOTE_SERVER_VAR,
 	SWITCH_CHANNELS_BETWEEN_WINDOWS_VAR,
 	TERM_DOES_BRIGHT_BLINK_VAR,
-	TRANSLATION_VAR,
-	TRANSLATION_PATH_VAR,
+	TMUX_OPTIONS_VAR,
 	USER_INFORMATION_VAR,
 	WORD_BREAK_VAR,
 	WSERV_PATH_VAR,
