@@ -1,11 +1,10 @@
-/* $EPIC: alias.c,v 1.88 2010/02/18 07:45:44 jnelson Exp $ */
 /*
  * alias.c -- Handles the whole kit and caboodle for aliases.
  *
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  1997, 2003 EPIC Software Labs
+ * Copyright 1997, 2014 EPIC Software Labs
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -1464,7 +1463,7 @@ static Symbol *	lookup_symbol (const char *name)
 static Symbol *	find_local_alias (const char *orig_name, SymbolSet **list)
 {
 	Symbol 	*alias = NULL;
-	int 	c = wind_index;
+	int 	c;
 	const char 	*ptr;
 	int 	implicit = -1;
 	int	function_return = 0;
@@ -1834,6 +1833,8 @@ static void	list_var_alias (const char *orig_name)
 		}
 	    }
 	}
+
+	new_free(&name);
 }
 
 /*
@@ -2167,6 +2168,7 @@ static PMATCH_SYMBOL(any_symbol, name)
 char **	get_subarray_elements (const char *orig_root, int *howmany, int type)
 {
 	SymbolSet *as;		/* XXXX */
+	Symbol *s;
 	int pos, cnt, max;
 	int cmp = 0;
 	char **matches = NULL;
@@ -2187,15 +2189,24 @@ char **	get_subarray_elements (const char *orig_root, int *howmany, int type)
 
 	for (cnt = 0; cnt < max; cnt++, pos++)
 	{
-		end = strcspn(ARRAY_ITEM(as, pos)->name + cmp, ".");
-		if (last && !my_strnicmp(ARRAY_ITEM(as, pos)->name, last, cmp + end))
+		s = (Symbol *)ARRAY_ITEM(as, pos);
+
+		end = strcspn(s->name + cmp, ".");
+		if (last && !my_strnicmp(s->name, last, cmp + end))
 			continue;
+
+		/* Must have an entry for the correct type */
+		if (type == COMMAND_ALIAS && s->user_command == NULL)
+			continue;
+		if (type == VAR_ALIAS && s->user_variable == NULL)
+			continue;
+
 		if (*howmany >= matches_size)
 		{
 			matches_size = *howmany + 5;
 			RESIZE(matches, char *, matches_size + 1);
 		}
-		matches[*howmany] = malloc_strndup(ARRAY_ITEM(as, pos)->name, cmp + end);
+		matches[*howmany] = malloc_strndup(s->name, cmp + end);
 		last = matches[*howmany];
 		*howmany += 1;
 	}
@@ -2455,45 +2466,32 @@ void	unlock_stack_frame	(void)
 
 void 	dump_call_stack 	(void)
 {
-	int my_wind_index = wind_index;
-	if (wind_index >= 0)
-	{
-		yell("Call stack");
-		while (my_wind_index--)
-			yell("[%3d] %s", my_wind_index, call_stack[my_wind_index].current);
-		yell("End of call stack");
-	}
+	int i;
+
+	yell("Call stack");
+	for (i = wind_index; i >= 0; i--)
+		yell("[%3d] %s", i, call_stack[i].current ? call_stack[i].current : "<async>");
+	yell("End of call stack");
 }
 
 void 	panic_dump_call_stack 	(void)
 {
-	int my_wind_index = wind_index;
-	fprintf(stderr, "Call stack\n");
 	if (wind_index >= 0)
 	{
-		while (my_wind_index--)
-			fprintf(stderr, "[%3d] %s\n", my_wind_index, call_stack[my_wind_index].current);
+		int i;
+
+		fprintf(stderr, "Call stack\n");
+		for (i = wind_index; i >= 0; i--)
+			fprintf(stderr, "[%3d] %s\n", i, 
+				call_stack[i].current ? call_stack[i].current : "<async>");
+		fprintf(stderr, "End of call stack\n");
 	}
 	else
 		fprintf(stderr, "Stack is corrupted [wind_index is %d], sorry.\n",
 			wind_index);
-	fprintf(stderr, "End of call stack\n");
 }
 
 
-/*
- * You may NOT call this unless youre about to exit.
- * If you do (call this when youre not about to exit), and you do it more 
- * than a few times, max_wind will get absurdly large.  So dont do it.
- *
- * XXXX - this doesnt clean up everything -- but do i care?
- */
-void 	destroy_call_stack 	(void)
-{
-	wind_index = -1;
-	new_free((char **)&call_stack);
-}
-
 /************************* DIRECT VARIABLE EXPANSION ************************/
 /*
  * get_variable: This returns the rvalue of the symbol "str".  
@@ -2543,8 +2541,9 @@ static char *	get_variable_with_args (const char *str, const char *args)
 	if ((local != -1) && (alias = find_local_alias(name, NULL)))
 		copy = 1, ret = alias->user_variable;
 	else if (local == 1)
-		;
-	else if ((alias = lookup_symbol(name)) != NULL)
+		(void) 0;
+
+	if (ret == NULL && ((alias = lookup_symbol(name)) != NULL))
 	{
 		if (alias->user_variable)
 			copy = 1, ret = alias->user_variable;
@@ -2554,14 +2553,19 @@ static char *	get_variable_with_args (const char *str, const char *args)
 			copy = 0, ret = make_string_var_bydata(alias->builtin_variable->type, alias->builtin_variable->data);
 	}
 /*
-	else if ((ret = make_string_var(str)))
+	if (ret == NULL && (ret = make_string_var(str)))
 		copy = 0;
 */
-	else
+
+	if (ret == NULL)
 		copy = 1, ret = getenv(str);
 
-	if (x_debug & DEBUG_UNKNOWN && ret == NULL)
-		yell("Variable lookup to non-existant assign [%s]", name);
+	if (ret == NULL)
+	{
+		copy = 0, ret = malloc_strdup(empty_string);
+		if (x_debug & DEBUG_UNKNOWN)
+		    yell("Variable lookup to non-existant assign [%s]", name);
+	}
 
 	new_free(&freep);
 	return (copy ? malloc_strdup(ret) : ret);
