@@ -1,4 +1,3 @@
-/* $EPIC: compat.c,v 1.36 2009/06/18 04:42:58 jnelson Exp $ */
 /*
  * Everything that im not directly responsible for I put in here.  Almost
  * all of this stuff is either borrowed from somewhere else (for you poor
@@ -1052,8 +1051,10 @@ static void	bsd_arc4_stir (ARC4 *as)
 
 	gettimeofday(&rdat.tv, NULL);
 	rdat.pid = getpid();
-	if ((fd = open("/dev/urandom", O_RDONLY, 0)) >= 0) {
-		read(fd, rdat.rnd, sizeof(rdat.rnd));
+	if ((fd = open("/dev/urandom", O_RDONLY, 0)) >= 0) 
+	{
+		if (read(fd, rdat.rnd, sizeof(rdat.rnd)) <= 0)
+			yell("Read from /dev/urandom failed.  Bummer.");
 		close(fd);
 	}
 	/* 
@@ -1306,7 +1307,7 @@ int	unsetenv (const char *name)
 # if defined(realpath)
 #  undef realpath
 # endif
-char *	my_realpath (const char *pathname, char resolved_path[MAXPATHLEN])
+char *	my_realpath (const char *pathname, char resolved_path[PATH_MAX])
 {
 	char *mypath;
 	char *rest;
@@ -1333,8 +1334,8 @@ char *	my_realpath (const char *pathname, char resolved_path[MAXPATHLEN])
 		return NULL;
 
 	/* And put the basename back on the result. */
-	strlcat(resolved_path, "/", MAXPATHLEN);
-	strlcat(resolved_path, rest, MAXPATHLEN);
+	strlcat(resolved_path, "/", PATH_MAX);
+	strlcat(resolved_path, rest, PATH_MAX);
 	return resolved_path;
 }
 #endif
