@@ -1,11 +1,10 @@
-/* $EPIC: ctcp.c,v 1.34 2004/11/10 03:20:35 jnelson Exp $ */
 /*
  * ctcp.c:handles the client-to-client protocol(ctcp). 
  *
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  1993, 2003 EPIC Software Labs
+ * Copyright 1993, 2014 EPIC Software Labs
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -53,9 +52,8 @@
 #include "status.h"
 #include "vars.h"
 #include "window.h"
-#include "if.h"
+#include "ifcmd.h"
 #include "flood.h"
-#include "notice.h"
 #include "words.h"
 
 #include <pwd.h>
@@ -68,7 +66,7 @@
 #define CTCP_INLINE	2	/* Expands to an inline value */
 #define CTCP_NOLIMIT	4	/* Limit of one per privmsg. */
 #define CTCP_TELLUSER	8	/* Tell the user about it. */
-
+#define CTCP_NORECODE	16	/* Don't recode message: ctcp func handles */
 static	int 	split_CTCP (char *, char *, char *);
 
 /*
@@ -92,30 +90,27 @@ typedef	struct _CtcpEntry
 	CTCP_Handler 	repl;	/* Function that is called for reply */
 }	CtcpEntry;
 
-/* forward declarations for the built in CTCP functions */
-static	char	*do_sed 	(CtcpEntry *, const char *, const char *, char *);
-static	char	*do_version 	(CtcpEntry *, const char *, const char *, char *);
-static	char	*do_clientinfo 	(CtcpEntry *, const char *, const char *, char *);
-static	char	*do_ping 	(CtcpEntry *, const char *, const char *, char *);
-static	char	*do_echo 	(CtcpEntry *, const char *, const char *, char *);
-static	char	*do_userinfo 	(CtcpEntry *, const char *, const char *, char *);
-static	char	*do_finger 	(CtcpEntry *, const char *, const char *, char *);
-static	char	*do_time 	(CtcpEntry *, const char *, const char *, char *);
-static	char	*do_atmosphere 	(CtcpEntry *, const char *, const char *, char *);
-static	char	*do_dcc 	(CtcpEntry *, const char *, const char *, char *);
-static	char	*do_utc 	(CtcpEntry *, const char *, const char *, char *);
-static	char	*do_dcc_reply 	(CtcpEntry *, const char *, const char *, char *);
-static	char	*do_ping_reply 	(CtcpEntry *, const char *, const char *, char *);
+#define CTCP_HANDLER(x) \
+static char * x (CtcpEntry *ctcp, Char *from, Char *to, char *cmd)
 
+/* forward declarations for the built in CTCP functions */
+CTCP_HANDLER(do_crypto);
+CTCP_HANDLER(do_version);
+CTCP_HANDLER(do_clientinfo);
+CTCP_HANDLER(do_ping);
+CTCP_HANDLER(do_echo);
+CTCP_HANDLER(do_userinfo);
+CTCP_HANDLER(do_finger);
+CTCP_HANDLER(do_time);
+CTCP_HANDLER(do_atmosphere);
+CTCP_HANDLER(do_dcc);
+CTCP_HANDLER(do_utc);
+CTCP_HANDLER(do_dcc_reply);
+CTCP_HANDLER(do_ping_reply);
 
 static CtcpEntry ctcp_cmd[] =
 {
-	{ "SED",	CTCP_SED, 	CTCP_INLINE | CTCP_NOLIMIT,
-		"contains simple_encrypted_data",
-		do_sed, 	do_sed },
-	{ "UTC",	CTCP_UTC, 	CTCP_INLINE | CTCP_NOLIMIT,
-		"substitutes the local timezone",
-		do_utc, 	do_utc },
+	/* The most common ones */
 	{ "ACTION",	CTCP_ACTION, 	CTCP_SPECIAL | CTCP_NOLIMIT,
 		"contains action descriptions for atmosphere",
 		do_atmosphere, 	do_atmosphere },
@@ -125,6 +120,47 @@ static CtcpEntry ctcp_cmd[] =
 	{ "VERSION",	CTCP_VERSION,	CTCP_REPLY | CTCP_TELLUSER,
 		"shows client type, version and environment",
 		do_version, 	NULL },
+
+	/* Common ones to people using strong crypto */
+	{ "AESSHA256-CBC",	CTCP_AESSHA256,	
+			CTCP_INLINE | CTCP_NOLIMIT | CTCP_NORECODE,
+		"transmit aes256-cbc ciphertext using a sha256 key",
+		do_crypto, 	do_crypto },
+	{ "AES256-CBC",		CTCP_AES256,	
+			CTCP_INLINE | CTCP_NOLIMIT | CTCP_NORECODE,
+		"transmit aes256-cbc ciphertext",
+		do_crypto, 	do_crypto },
+	{ "CAST128ED-CBC",	CTCP_CAST5, 	
+			CTCP_INLINE | CTCP_NOLIMIT | CTCP_NORECODE,
+		"transmit cast5-cbc ciphertext",
+		do_crypto, 	do_crypto},
+	{ "BLOWFISH-CBC",	CTCP_BLOWFISH,	
+			CTCP_INLINE | CTCP_NOLIMIT | CTCP_NORECODE,
+		"transmit blowfish-cbc ciphertext",
+		do_crypto, 	do_crypto },
+	{ "FISH",		CTCP_FISH,	
+			CTCP_INLINE | CTCP_NOLIMIT | CTCP_NORECODE,
+		"transmit FiSH (blowfish-ecb with sha256'd key) ciphertext",
+		do_crypto, 	do_crypto },
+	{ "SED",		CTCP_SED, 	
+			CTCP_INLINE | CTCP_NOLIMIT | CTCP_NORECODE,
+		"transmit simple_encrypted_data ciphertext",
+		do_crypto, 	do_crypto },
+	{ "SEDSHA",		CTCP_SEDSHA, 	
+			CTCP_INLINE | CTCP_NOLIMIT | CTCP_NORECODE,
+		"transmit simple_encrypted_data ciphertext using a sha256 key",
+		do_crypto, 	do_crypto },
+
+	/* The uncommon ones */
+	{ "PING", 	CTCP_PING, 	CTCP_REPLY | CTCP_TELLUSER,
+		"returns the arguments it receives",
+		do_ping, 	do_ping_reply },
+	{ "ECHO", 	CTCP_ECHO, 	CTCP_REPLY | CTCP_TELLUSER,
+		"returns the arguments it receives",
+		do_echo, 	NULL },
+	{ "UTC",	CTCP_UTC, 	CTCP_INLINE | CTCP_NOLIMIT,
+		"substitutes the local timezone",
+		do_utc, 	do_utc },
 	{ "CLIENTINFO",	CTCP_CLIENTINFO,CTCP_REPLY | CTCP_TELLUSER,
 		"gives information about available CTCP commands",
 		do_clientinfo, 	NULL },
@@ -140,12 +176,6 @@ static CtcpEntry ctcp_cmd[] =
 	{ "TIME",	CTCP_TIME, 	CTCP_REPLY | CTCP_TELLUSER,
 		"tells you the time on the user's host",
 		do_time, 	NULL },
-	{ "PING", 	CTCP_PING, 	CTCP_REPLY | CTCP_TELLUSER,
-		"returns the arguments it receives",
-		do_ping, 	do_ping_reply },
-	{ "ECHO", 	CTCP_ECHO, 	CTCP_REPLY | CTCP_TELLUSER,
-		"returns the arguments it receives",
-		do_echo, 	NULL },
 	{ NULL,		CTCP_CUSTOM,	CTCP_REPLY | CTCP_TELLUSER,
 		NULL,
 		NULL, NULL }
@@ -167,25 +197,22 @@ int     sed = 0;
  */
 int	in_ctcp_flag = 0;
 
-#define CTCP_HANDLER(x) \
-	static char * x (CtcpEntry *ctcp, const char *from, const char *to, char *cmd)
-
-
 
 /**************************** CTCP PARSERS ****************************/
 
 /********** INLINE EXPANSION CTCPS ***************/
 /*
- * do_sed: Performs the Simple Encrypted Data trasfer for ctcp.  Returns in a
- * malloc string the decryped message (if a key is set for that user) or the
- * text "[ENCRYPTED MESSAGE]" 
+ * do_crypto: Performs strong decryption for ctcp.  Returns in a malloc 
+ * string the decryped message (if a key is set for that user) or the text 
+ * "[ENCRYPTED MESSAGE]" 
  */
-CTCP_HANDLER(do_sed)
+CTCP_HANDLER(do_crypto)
 {
 	Crypt	*key = NULL;
 	const char	*crypt_who;
 	char 	*tofrom;
-	char	*ret = NULL, *ret2 = NULL;
+	char	*ret = NULL;
+	char 	*extra = NULL;
 
 	if (*from == '=')		/* DCC CHAT message */
 		crypt_who = from;
@@ -197,47 +224,82 @@ CTCP_HANDLER(do_sed)
 	tofrom = malloc_strdup3(to, ",", from);
 	malloc_strcat2_c(&tofrom, "!", FromUserHost, NULL);
 
-	if ((key = is_crypted(tofrom)) ||
-	    (key = is_crypted(crypt_who)))
+	if ((key = is_crypted(tofrom, from_server, ctcp->id)) ||
+	    (key = is_crypted(crypt_who, from_server, ctcp->id)))
 		ret = decrypt_msg(cmd, key);
 
 	new_free(&tofrom);
 
-	if (!key || !ret) {
+	/*
+	 * Key would be NULL if someone sent us a rogue encrypted
+	 * message (ie, we don't have a password).  Ret should never
+	 * be NULL (but we can be defensive against the future).
+	 * In either case, something went seriously wrong.
+	 */
+	if (!key || !ret) 
+	{
+		if (ret)
+			new_free(&ret);
+
 		sed = 2;
-		malloc_strcpy(&ret2, "[ENCRYPTED MESSAGE]");
-	} else if (!*ret) {
-		sed = 2;
-		malloc_strcpy(&ret2, "[ENCRYPTED MESSAGE - BAD KEY?]");
-	} else {
-		/* 
-		 * There might be a CTCP message in there,
-		 * so we see if we can find it.
-		 */
-		if (get_server_doing_privmsg(from_server))
-			ret2 = malloc_strdup(do_ctcp(from, to, ret));
-		else if (get_server_doing_notice(from_server))
-			ret2 = malloc_strdup(do_notice_ctcp(from, to, ret));
-		sed = 1;
+		malloc_strcpy(&ret, "[ENCRYPTED MESSAGE]");
+		return ret;
+	} 
+
+
+	/*
+	 * NOW WE HANDLE THE DECRYPTED MESSAGE....
+	 */
+
+	/*
+	 * CTCP messages can be recursive (ie, a decrypted msg
+	 * might yield another CTCP message), and so we must not
+	 * recode until we have removed any sub-ctcps!
+	 */
+	if (get_server_doing_privmsg(from_server))
+		extra = malloc_strdup(do_ctcp(from, to, ret));
+	else if (get_server_doing_notice(from_server))
+		extra = malloc_strdup(do_notice_ctcp(from, to, ret));
+	else
+	{
+		extra = ret;
+		ret = NULL;
 	}
 
 	new_free(&ret);
-	return ret2;
+	ret = extra;
+	extra = NULL;
+
+	/*
+	 * What we're left with is just the plain part of the CTCP.
+	 * In rfc1459_any_to_utf8(), CTCP messages are specifically
+	 * detected and ignored [because recoding binary data will
+	 * corrupt the data].  But that does not mean the message
+	 * doesn't need decoding -- it just needs to be done after
+	 * the message is decrypted.
+	 */
+	inbound_recode(from, from_server, to, ret, &extra);
+
+	/*
+	 * If a recoding actually occurred, free the source string
+	 * and then use the decoded string going forward.
+	 */
+	if (extra)
+	{
+		new_free(&ret);
+		ret = extra;
+	}
+
+	sed = 1;
+	return ret;
 }
 
 CTCP_HANDLER(do_utc)
 {
-	long	sec;
-
 	if (!cmd || !*cmd)
 		return malloc_strdup(empty_string);
 
-	/* Check for shenanigans */
-	sec = my_atol(cmd);
-	if (sec < (long)INT_MIN * 2L || sec > (long)INT_MAX * 2L)
-		return malloc_strdup(empty_string);
-
-	return malloc_strdup(my_ctime(sec));
+	return malloc_strdup(my_ctime(my_atol(cmd)));
 }
 
 
@@ -249,16 +311,24 @@ CTCP_HANDLER(do_utc)
  */
 CTCP_HANDLER(do_atmosphere)
 {
-	const char	*old_message_from;
-	int	old_message_level;
+	int	l;
+	int	flag, fflag;
 
 	if (!cmd || !*cmd)
 		return NULL;
 
-	save_message_from(&old_message_from, &old_message_level);
+	/* Xavier mentioned that we should allow /ignore #chan action */
+	flag = check_ignore_channel(from, FromUserHost, to, LEVEL_ACTION);
+	fflag = new_check_flooding(from, FromUserHost, 
+					is_channel(to) ? to : NULL,
+					cmd, LEVEL_ACTION);
+
+	if (flag == IGNORED || fflag == 1)
+		return NULL;
+
 	if (is_channel(to))
 	{
-		message_from(to, LOG_ACTION);
+		l = message_from(to, LEVEL_ACTION);
 		if (do_hook(ACTION_LIST, "%s %s %s", from, to, cmd))
 		{
 			if (is_current_channel(to, from_server))
@@ -269,12 +339,12 @@ CTCP_HANDLER(do_atmosphere)
 	}
 	else
 	{
-		message_from(from, LOG_ACTION);
+		l = message_from(from, LEVEL_ACTION);
 		if (do_hook(ACTION_LIST, "%s %s %s", from, to, cmd))
 			put_it("*> %s %s", from, cmd);
 	}
 
-	restore_message_from(old_message_from, old_message_level);
+	pop_message_from(l);
 	return NULL;
 }
 
@@ -296,8 +366,8 @@ CTCP_HANDLER(do_dcc)
 
 	if     (!(type = next_arg(cmd, &cmd)) ||
 		!(description = (get_int_var(DCC_DEQUOTE_FILENAMES_VAR)
-			 ? new_next_arg(cmd, &cmd)
-			 : next_arg(cmd, &cmd))) ||
+				? new_next_arg(cmd, &cmd)
+				: next_arg(cmd, &cmd))) ||
 		!(inetaddr = next_arg(cmd, &cmd)) ||
 		!(port = next_arg(cmd, &cmd)))
 			return NULL;
@@ -370,30 +440,24 @@ CTCP_HANDLER(do_version)
 	 */
 #if defined(HAVE_UNAME) && !defined(UNAME_HACK)
 	struct utsname un;
-	const char	*the_unix,
-			*the_version;
+	const char	*the_unix;
 
 	if (uname(&un) < 0)
-	{
-		the_version = empty_string;
 		the_unix = "unknown";
-	}
 	else
-	{
-		the_version = un.release;
 		the_unix = un.sysname;
-	}
 
+	/* We no longer show the detailed version of your OS. */
 	send_ctcp(CTCP_NOTICE, from, CTCP_VERSION, 
-			"ircII %s %s %s - %s", 
-			irc_version, the_unix, the_version,
-			(tmp = get_string_var(CLIENTINFO_VAR)) ? 
+			"ircII %s %s - %s", 
+			irc_version, the_unix, 
+			(tmp = get_string_var(CLIENT_INFORMATION_VAR)) ? 
 				tmp : IRCII_COMMENT);
 #else
 	send_ctcp(CTCP_NOTICE, from, CTCP_VERSION, 
 			"ircII %s *IX - %s", 
 			irc_version,
-			(tmp = get_string_var(CLIENTINFO_VAR)) ? 
+			(tmp = get_string_var(CLIENT_INFORMATION_VAR)) ? 
 				tmp : IRCII_COMMENT);
 #endif
 	return NULL;
@@ -413,7 +477,7 @@ CTCP_HANDLER(do_userinfo)
 	char *tmp;
 
 	send_ctcp(CTCP_NOTICE, from, CTCP_USERINFO, "%s", 
-		(tmp = get_string_var(USERINFO_VAR)) ? tmp : "<No User Information>");
+		(tmp = get_string_var(USER_INFORMATION_VAR)) ? tmp : "<No User Information>");
 	return NULL;
 }
 
@@ -464,7 +528,7 @@ CTCP_HANDLER(do_finger)
 #define GECOS_DELIMITER ','
 #endif
 
-#if !defined(I_DONT_TRUST_MY_USERS)
+#if defined(ALLOW_USER_SPECIFIED_LOGIN)
 	if ((ctcpuser = getenv("IRCUSER"))) 
 		strlcpy(userbuff, ctcpuser, sizeof userbuff);
 	else
@@ -476,7 +540,7 @@ CTCP_HANDLER(do_finger)
 			strlcpy(userbuff, "epic-user", sizeof userbuff);
 	}
 
-#if !defined(I_DONT_TRUST_MY_USERS)
+#if defined(ALLOW_USER_SPECIFIED_LOGIN)
 	if ((ctcpfinger = getenv("IRCFINGER"))) 
 		strlcpy(gecosbuff, ctcpfinger, sizeof gecosbuff);
 	else
@@ -546,7 +610,7 @@ CTCP_HANDLER(do_ping_reply)
 
 	tsec = t.tv_sec - orig;
 
-	if ((ptr = sindex(cmd, " .")))
+	if ((ptr = strchr(cmd, ' ')) || (ptr = strchr(cmd, '.')))
 	{
 		*ptr++ = 0;
 		tusec = t.tv_usec - my_atol(ptr);
@@ -594,7 +658,6 @@ char *	do_ctcp (const char *from, const char *to, char *str)
 {
 	int 	flag;
 	int	fflag;
-	int	lastlog_level;
 	char 	local_ctcp_buffer [BIG_BUFFER_SIZE + 1],
 		the_ctcp          [IRCD_BUFFER_SIZE + 1],
 		last              [IRCD_BUFFER_SIZE + 1];
@@ -604,6 +667,8 @@ char *	do_ctcp (const char *from, const char *to, char *str)
 	char	*ptr = NULL;
 	int	allow_ctcp_reply = 1;
 static	time_t	last_ctcp_parsed = 0;
+	int	l;
+	char *	extra = NULL;
 
 	int delim_char = charcount(str, CTCP_DELIM_CHAR);
 
@@ -612,14 +677,13 @@ static	time_t	last_ctcp_parsed = 0;
 	if (delim_char > 8)
 		allow_ctcp_reply = 0;	/* Historical limit of 4 CTCPs */
 
-	flag = check_ignore_channel(from, FromUserHost, to, IGNORE_CTCPS);
+	flag = check_ignore_channel(from, FromUserHost, to, LEVEL_CTCP);
 	fflag = new_check_flooding(from, FromUserHost, is_channel(to) ? to : NULL,
-						str, CTCP_FLOOD); 
+						str, LEVEL_CTCP);
 
 	in_ctcp_flag++;
 	strlcpy(local_ctcp_buffer, str, sizeof(local_ctcp_buffer) - 2);
 
-	lastlog_level = set_lastlog_msg_level(LOG_CTCP);
 	for (;;strlcat(local_ctcp_buffer, last, sizeof(local_ctcp_buffer) - 2))
 	{
 		if (split_CTCP(local_ctcp_buffer, the_ctcp, last))
@@ -700,13 +764,13 @@ static	time_t	last_ctcp_parsed = 0;
 		if (ctcp_argument)
 			*ctcp_argument++ = 0;
 		else
-			ctcp_argument = empty_string;
+			ctcp_argument = endstr(the_ctcp);
 
 		/* Set up the window level/logging */
 		if (im_on_channel(to, from_server))
-			message_from(to, LOG_CTCP);
+			l = message_from(to, LEVEL_CTCP);
 		else
-			message_from(from, LOG_CTCP);
+			l = message_from(from, LEVEL_CTCP);
 
 		/*
 		 * Then we look for the correct CTCP.
@@ -724,24 +788,52 @@ static	time_t	last_ctcp_parsed = 0;
 			 * Offer it to the user.
 			 * Maybe they know what to do with it.
 			 */
-			if (do_hook(CTCP_LIST, "%s %s %s %s", from, to, 
-						ctcp_command, ctcp_argument))
+			if (do_hook(CTCP_REQUEST_LIST, "%s %s %s %s",
+				from, to, ctcp_command, ctcp_argument))
 			{
-				if (get_int_var(VERBOSE_CTCP_VAR))
+			    if (do_hook(CTCP_LIST, "%s %s %s %s", from, to, 
+						ctcp_command, ctcp_argument))
+			    {
 				    say("Unknown CTCP %s from %s to %s: %s%s",
 					ctcp_command, from, to, 
 					*ctcp_argument ? ": " : empty_string, 
 					ctcp_argument);
+			    }
 			}
 			time(&last_ctcp_parsed);
 			allow_ctcp_reply = 0;
+			pop_message_from(l);
 			continue;
 		}
 
+		/*
+		 * rfc1459_any_to_utf8 specifically ignores CTCPs, because
+		 * recoding binary data (such as an encrypted message) would
+		 * corrupt the message.  
+		 *
+		 * So some CTCPs are "recodable" and some are not.
+		 *
+		 * The CTCP_NORECODE is set for any CTCPs which are NOT
+		 * to be recoded prior to handling.  These are the encryption
+		 * CTCPS.
+		 *
+		 * All other CTCPs have not been recoded by the time they
+		 * reach here, so we must do it here!
+		 */
+		if (!(ctcp_cmd[i].flag & CTCP_NORECODE))
+		{
+		   /*
+		    * We must recode to UTF8
+		    */
+		   inbound_recode(from, from_server, to, ctcp_argument, &extra);
+		   if (extra)
+			ctcp_argument = extra;
+		}
+
 		/* 
 		 * We did find it.  Acknowledge it.
 		 */
-		ptr = empty_string;
+		ptr = NULL;
 		if (do_hook(CTCP_REQUEST_LIST, "%s %s %s %s",
 				from, to, ctcp_command, ctcp_argument))
 		{
@@ -772,7 +864,7 @@ static	time_t	last_ctcp_parsed = 0;
 		 * If its an ``INLINE'' CTCP, we paste it back in.
 		 */
 		if (ctcp_cmd[i].flag & CTCP_INLINE)
-			strlcat(local_ctcp_buffer, ptr, sizeof local_ctcp_buffer);
+			strlcat(local_ctcp_buffer, ptr ? ptr : empty_string, sizeof local_ctcp_buffer);
 
 		/* 
 		 * If its ``INTERESTING'', tell the user.
@@ -784,8 +876,6 @@ static	time_t	last_ctcp_parsed = 0;
 		    if (do_hook(CTCP_LIST, "%s %s %s %s", 
 				from, to, ctcp_command, ctcp_argument))
 		    {
-			if (get_int_var(VERBOSE_CTCP_VAR)) 
-			{
 			    if (is_me(from_server, to))
 				say("CTCP %s from %s%s%s", 
 					ctcp_command, from, 
@@ -796,17 +886,13 @@ static	time_t	last_ctcp_parsed = 0;
 					ctcp_command, from, to, 
 					*ctcp_argument ? ": " : empty_string, 
 					ctcp_argument);
-			}
 		    }
 		}
-		if (ptr != empty_string)
-			new_free(&ptr);
+		new_free(&extra);
+		new_free(&ptr);
+		pop_message_from(l);
 	}
 
-	/* Reset the window level/logging */
-	set_lastlog_msg_level(lastlog_level);
-	message_from(NULL, LOG_CRAP);
-
 	in_ctcp_flag--;
 
 	/* 
@@ -825,7 +911,6 @@ static	time_t	last_ctcp_parsed = 0;
 char *	do_notice_ctcp (const char *from, const char *to, char *str)
 {
 	int 	flag;
-	int	lastlog_level;
 	char 	local_ctcp_buffer [BIG_BUFFER_SIZE + 1],
 		the_ctcp          [IRCD_BUFFER_SIZE + 1],
 		last              [IRCD_BUFFER_SIZE + 1];
@@ -834,6 +919,7 @@ char *	do_notice_ctcp (const char *from, const char *to, char *str)
 	int	i;
 	char	*ptr;
 	int	allow_ctcp_reply = 1;
+	int	l;
 
 	int delim_char = charcount(str, CTCP_DELIM_CHAR);
 
@@ -842,12 +928,12 @@ char *	do_notice_ctcp (const char *from, const char *to, char *str)
 	if (delim_char > 8)
 		allow_ctcp_reply = 0;	/* Ignore all the CTCPs. */
 
-	flag = check_ignore_channel(from, FromUserHost, to, IGNORE_CTCPS);
-	if (!in_ctcp_flag)
-		in_ctcp_flag = -1;
-	strlcpy(local_ctcp_buffer, str, IRCD_BUFFER_SIZE - 2);
+	/* We handle ignore, but not flooding (obviously) */
+	flag = check_ignore_channel(from, FromUserHost, to, LEVEL_CTCP);
+	in_ctcp_flag++;
+	strlcpy(local_ctcp_buffer, str, sizeof(local_ctcp_buffer) - 2);
 
-	for (;;strlcat(local_ctcp_buffer, last, sizeof local_ctcp_buffer))
+	for (;;strlcat(local_ctcp_buffer, last, sizeof(local_ctcp_buffer) - 2))
 	{
 		if (split_CTCP(local_ctcp_buffer, the_ctcp, last))
 			break;		/* All done! */
@@ -856,7 +942,8 @@ char *	do_notice_ctcp (const char *from, const char *to, char *str)
 			continue;	/* Empty requests are ignored */
 
 		/*
-		 * Apply sanity rules
+		 * The logic of all this is essentially the same as 
+		 * do_ctcp
 		 */
 
 		if (!allow_ctcp_reply)
@@ -870,8 +957,11 @@ char *	do_notice_ctcp (const char *from, const char *to, char *str)
 			continue;
 		}
 
+		/* But we don't check ctcp flooding (obviously) */
+
 		/* Global messages -- just drop the CTCP */
-		if (*to == '$' || (*to == '#' && !im_on_channel(to, from_server)))
+		if (*to == '$' || (is_channel(to) && 
+					!im_on_channel(to, from_server)))
 		{
 			allow_ctcp_reply = 0;
 			continue;
@@ -880,20 +970,24 @@ char *	do_notice_ctcp (const char *from, const char *to, char *str)
 
 		/*
 		 * Parse CTCP message
-		 * CTCP spec says word delim MUST be space.
+		 * CTCP spec says word delim MUST be space
 		 */
 		ctcp_command = the_ctcp;
 		ctcp_argument = strchr(the_ctcp, ' ');
 		if (ctcp_argument)
 			*ctcp_argument++ = 0;
 		else
-			ctcp_argument = empty_string;
+			ctcp_argument = endstr(the_ctcp);
 
+		/* Set up the window level/logging */
+		if (is_channel(to))
+			l = message_from(to, LEVEL_CTCP);
+		else
+			l = message_from(from, LEVEL_CTCP);
 
 		/* 
 		 * Find the correct CTCP and run it.
 		 */
-
 		for (i = 0; i < NUMBER_OF_CTCPS; i++)
 			if (!strcmp(ctcp_command, ctcp_cmd[i].name))
 				break;
@@ -904,31 +998,32 @@ char *	do_notice_ctcp (const char *from, const char *to, char *str)
 		 */
 		if (i < NUMBER_OF_CTCPS && ctcp_cmd[i].repl)
 		{
-			if ((ptr = ctcp_cmd[i].repl(ctcp_cmd + i, from, to, ctcp_argument)))
-			{
-				strlcat(local_ctcp_buffer, ptr, sizeof local_ctcp_buffer);
-				new_free(&ptr);
-				continue;
-			}
+		    if ((ptr = ctcp_cmd[i].repl(ctcp_cmd + i, from, to, 
+						ctcp_argument)))
+		    {
+			strlcat(local_ctcp_buffer, ptr, 
+					sizeof local_ctcp_buffer);
+			new_free(&ptr);
+			pop_message_from(l);
+			continue;
+		    }
 		}
 
-		/* Set up the window level/logging */
-		lastlog_level = set_lastlog_msg_level(LOG_CTCP);
-		message_from(NULL, LOG_CTCP);
-
 		/* Toss it at the user.  */
-		if (do_hook(CTCP_REPLY_LIST, "%s %s %s %s", from, to, ctcp_command, ctcp_argument))
-			say("CTCP %s reply from %s: %s", ctcp_command, from, ctcp_argument);
-
-		/* Reset the window level/logging */
-		set_lastlog_msg_level(lastlog_level);
-
+		if (ctcp_cmd[i].flag & CTCP_TELLUSER)
+		{
+		    if (do_hook(CTCP_REPLY_LIST, "%s %s %s %s", 
+					from, to, ctcp_command, ctcp_argument))
+			say("CTCP %s reply from %s: %s", 
+					ctcp_command, from, ctcp_argument);
+		}
 		if (!(ctcp_cmd[i].flag & CTCP_NOLIMIT))
 			allow_ctcp_reply = 0;
+
+		pop_message_from(l);
 	}
 
-	if (in_ctcp_flag == -1)
-		in_ctcp_flag = 0;
+	in_ctcp_flag--;
 
 	/* 
 	 * local_ctcp_buffer is derived from 'str', so its always
@@ -960,12 +1055,16 @@ void	send_ctcp (int type, const char *to, int datatag, const char *format, ...)
 	char 	putbuf [BIG_BUFFER_SIZE + 1],
 		*putbuf2;
 	int	len;
+	int	l;
+	const char *pb;
+	char	*extra = NULL;
 
 	/* Make sure that the final \001 doesnt get truncated */
 	if ((len = IRCD_BUFFER_SIZE - (12 + strlen(to))) <= 0)
 		return;				/* Whatever. */
 	putbuf2 = alloca(len);
 
+	l = message_from(to, LEVEL_CTCP);
 	if (format)
 	{
 		va_list args;
@@ -973,13 +1072,17 @@ void	send_ctcp (int type, const char *to, int datatag, const char *format, ...)
 		vsnprintf(putbuf, BIG_BUFFER_SIZE, format, args);
 		va_end(args);
 
+		pb = outbound_recode(to, from_server, putbuf, &extra);
+
 		do_hook(SEND_CTCP_LIST, "%s %s %s %s", 
 				ctcp_type[type], to, 
-				ctcp_cmd[datatag].name, putbuf);
+				ctcp_cmd[datatag].name, pb);
 		snprintf(putbuf2, len, "%c%s %s%c", 
 				CTCP_DELIM_CHAR, 
-				ctcp_cmd[datatag].name, putbuf, 
+				ctcp_cmd[datatag].name, pb, 
 				CTCP_DELIM_CHAR);
+
+		new_free(&extra);
 	}
 	else
 	{
@@ -996,7 +1099,8 @@ void	send_ctcp (int type, const char *to, int datatag, const char *format, ...)
 	putbuf2[len - 2] = CTCP_DELIM_CHAR;
 	putbuf2[len - 1] = 0;
 
-	send_text(to, putbuf2, ctcp_type[type], 0);
+	send_text(from_server, to, putbuf2, ctcp_type[type], 0, 1);
+	pop_message_from(l);
 }
 
 
@@ -1048,3 +1152,13 @@ static int split_CTCP (char *raw_message, char *ctcp_dest, char *after_ctcp)
 	strlcpy(after_ctcp, ctcp_end, IRCD_BUFFER_SIZE - 1);
 	return 0;		/* All done! */
 }
+
+#if 0
+void    help_topics_ctcp (FILE *f)
+{
+        int     x;                                                              
+
+        for (x = 0; ctcp_cmd[x].name; x++)                            
+                fprintf(f, "ctcp %s\n", ctcp_cmd[x].name);
+}
+#endif
