@@ -1,7 +1,7 @@
 /*
  * elf.c: code for generalizing loading from just about anywhere
  *
- * Copyright  2007 EPIC Software Labs.
+ * Copyright 2007, 2012 EPIC Software Labs.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -129,7 +129,7 @@ static int	archive_fopen(struct epic_loadfile *elf, char *filename, const char *
         elf->a = archive_read_new();
         if (!archive_read_support_format_all(elf->a)) {
 
-            if ( !(ret = archive_read_open_file(elf->a, fname, 10240)) ) {
+            if ( !(ret = archive_read_open_filename(elf->a, fname, 10240)) ) {
                 if ( (strstr(safet, "/"))!=NULL ) { /* specific file provided */
                     if (!find_in_archive(elf->a, &elf->entry, extra, do_error))
                         return 0;
@@ -238,7 +238,7 @@ int	epic_fclose(struct epic_loadfile *elf)
 #ifdef HAVE_LIBARCHIVE
     else if ((elf->a)!=NULL) {
         archive_read_close(elf->a);
-        archive_read_finish(elf->a);
+        archive_read_free(elf->a);
         return 0;
     } 
 #endif
@@ -288,7 +288,7 @@ off_t	epic_stat(const char *filename, struct stat *buf)
         a = archive_read_new();
         archive_read_support_format_all(a);
 
-        if ( !(ret=archive_read_open_file(a, zipstr, 10240)) ) {
+        if ( !(ret=archive_read_open_filename(a, zipstr, 10240)) ) {
             if (scan) {
                 if (!(find_in_archive(a, &entry, zip, 0)) ) {
                     return -1;
@@ -299,7 +299,7 @@ off_t	epic_stat(const char *filename, struct stat *buf)
                     buf->st_mode=33188;
 
                     archive_read_close(a);
-                    archive_read_finish(a);
+                    archive_read_free(a);
                     return 0;
                 }
             }
@@ -323,7 +323,7 @@ static int	find_in_archive(struct archive *a, struct archive_entry **entry, cons
         ret = archive_read_next_header(a, entry);
         if (ret == ARCHIVE_EOF) {
             archive_read_close(a);
-            archive_read_finish(a);
+            archive_read_free(a);
             return 0;
         }
         if (ret != ARCHIVE_OK) {
@@ -331,7 +331,7 @@ static int	find_in_archive(struct archive *a, struct archive_entry **entry, cons
                 yell("%s", archive_error_string(a));
             }
             archive_read_close(a);
-            archive_read_finish(a);
+            archive_read_free(a);
             return 0;
         }
 
@@ -389,11 +389,11 @@ size_t	slurp_elf_file (struct epic_loadfile *elf, char **file_contents, off_t *f
 		next_byte++;
 	}
 
-	/* Just for laughs, zero terminate it so it's a C string */
-	size = next_byte + 1;
-	RESIZE(*file_contents, char, size);
-	(*file_contents)[next_byte] = 0;
+	/* The previous epic_fgetc() resulted in EOF so we got to back up */
+	next_byte--;
 
+	/* Just for laughs, zero terminate it so it's a C string */
+	(*file_contents)[next_byte] = 0;
 	*file_contents_size = next_byte;
 	return next_byte;
 }
@@ -418,7 +418,7 @@ int	string_fgetc(const char **file_contents, off_t *file_contents_size)
 	return retval;
 }
 
-char *	string_fgets(char *buffer, size_t buffer_size, const char **file_contents, off_t *file_contents_size)
+off_t	string_fgets(char *buffer, size_t buffer_size, const char **file_contents, off_t *file_contents_size)
 {
 	size_t	offset = 0;
 	int	next_byte = 0;
@@ -430,9 +430,9 @@ char *	string_fgets(char *buffer, size_t buffer_size, const char **file_contents
 		if (next_byte == EOF)
 		{
 			if (!read_any_byte)
-				return NULL;
+				return 0;
 			else
-				return buffer;		/* All done! */
+				return offset;
 		}
 
 		read_any_byte = 1;
@@ -442,7 +442,7 @@ char *	string_fgets(char *buffer, size_t buffer_size, const char **file_contents
 		{
 			if (offset <= buffer_size)
 				buffer[offset++] = 0;
-			return buffer;
+			return offset;
 		}
 	}
 }
