@@ -1,4 +1,3 @@
-/* $EPIC: expr.c,v 1.40 2007/04/12 03:24:14 jnelson Exp $ */
 /*
  * expr.c -- The expression mode parser and the textual mode parser
  * #included by alias.c -- DO NOT DELETE
@@ -6,7 +5,7 @@
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  1993, 2003 EPIC Software Labs
+ * Copyright 1993, 2003 EPIC Software Labs
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,7 +39,7 @@
 /* Function decls */
 static	void	TruncateAndQuote (char **, const char *, ssize_t, const char *);
 static	char	*alias_special_char(char **, char *, const char *, char *);
-static	void	do_alias_string (char *, char *);
+static	void	do_alias_string (char *, const char *);
 
 char *alias_string = NULL;
 
@@ -140,8 +139,7 @@ static	char	*next_unit (char *str, const char *args, int stage)
 		*lastc,			/* used to denote end of token-set */
 		*tmp = NULL,		/* place to malloc_sprintf() into */
 		op;			/* the op were working on */
-	int	got_sloshed = 0,	/* If the last char was a slash */
-		display;
+	int	got_sloshed = 0;	/* If the last char was a slash */
 
 	char	*result1 = (char *) 0,	/* raw lefthand-side of operator */
 		*result2 = (char *) 0,	/* raw righthand-side of operator */
@@ -417,8 +415,6 @@ static	char	*next_unit (char *str, const char *args, int stage)
 		{
 			ssize_t span;
 
-			display = window_display;
-
 			if ((span = MatchingBracket(ptr + 1, '{', '}')) >= 0)
 				ptr2 = ptr + 1 + span;
 			else
@@ -1468,7 +1464,7 @@ char	*expand_alias	(const char *string, const char *args)
 		malloc_strcat_ues_c(&buffer, stuff, unescape, &buffclue);
 
 	if (get_int_var(DEBUG_VAR) & DEBUG_EXPANSIONS)
-		privileged_yell("Expanded [%s] to [%s]", string, buffer);
+		privileged_yell("Expanded " BOLD_TOG_STR "[" BOLD_TOG_STR "%s" BOLD_TOG_STR "]" BOLD_TOG_STR " to " BOLD_TOG_STR "[" BOLD_TOG_STR "%s" BOLD_TOG_STR "]" BOLD_TOG_STR, string, buffer);
 
 #if 0						/* Maybe a good idea later? */
 	if (!buffer)
@@ -1875,6 +1871,9 @@ static	void	TruncateAndQuote (char **buff, const char *add, ssize_t length, cons
 {
 	size_t	real_size;
 	char *	buffer;
+	char *	free_me = NULL;
+	int	justify;
+	int	pad;
 
 	if (!add)
 		panic(1, "add is NULL");
@@ -1889,17 +1888,14 @@ static	void	TruncateAndQuote (char **buff, const char *add, ssize_t length, cons
 	 *	If quote_em is NULL, then the value of "add" is
 	 *	   appended to "buff"
 	 */
-
 	if (length)
 	{
-		/*
-		 * No, the "alloca" space doesnt go away until the end
-		 * of the function, so yes, we can reference "add" after
-		 * the end of this block.
-		 */
-		real_size = abs(length) + 1;
-		buffer = alloca(real_size);
-		add = strformat(buffer, add, length, get_int_var(PAD_CHAR_VAR));
+		/* -1 is left justify, 1 is right justify */
+		justify = (length > 0) ? -1 : 1;
+		if (length < 0)
+			length = -length;
+		pad = get_int_var(PAD_CHAR_VAR);
+		add = free_me = fix_string_width(add, justify, pad, length, 1);
 	}
 	if (quote_em && add)	/* add shouldnt ever be NULL, though! */
 	{
@@ -1910,11 +1906,12 @@ static	void	TruncateAndQuote (char **buff, const char *add, ssize_t length, cons
 
 	if (buff)
 		malloc_strcat(buff, add);
-
+	if (free_me)
+		new_free(&free_me);
 	return;
 }
 
-static void	do_alias_string (char *unused, char *input)
+static void	do_alias_string (char *unused, const char *input)
 {
 	malloc_strcpy(&alias_string, input);
 }
