@@ -1,10 +1,9 @@
-/* $EPIC: if.c,v 1.41 2008/11/30 19:09:23 jnelson Exp $ */
 /*
  * if.c: the IF, WHILE, FOREACH, DO, FE, FEC, and FOR commands for IRCII 
  *
  * Copyright (c) 1991, 1994 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew R. Green.
- * Copyright  1993, 2003 EPIC Software Labs.
+ * Copyright 1993, 2003 EPIC Software Labs.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -303,6 +302,10 @@ BUILT_IN_COMMAND(whilecmd)
 		*body = NULL,
 		*newexp = NULL;
 	int 	whileval = !strcmp(command, "WHILE");
+	size_t	sigh;
+
+	if (!args)
+		return;
 
 	if (!subargs)
 		subargs = empty_string;
@@ -315,15 +318,29 @@ BUILT_IN_COMMAND(whilecmd)
 	exp = LOCAL_COPY(ptr);
 
 	if (!(ptr = next_expr_failok(&args, '{')))
-		ptr = args;
+	{
+		if (!(ptr = args))
+		{
+			my_error("WHILE: Missing body");
+			return;
+		}
+	}
+
 	body = LOCAL_COPY(ptr);
 
 	will_catch_break_exceptions++;
 	will_catch_continue_exceptions++;
-	newexp = alloca(strlen(exp) + 2);
+	sigh = strlen(exp) + 1;		/* Just to make clang shut up */
+	newexp = alloca(sigh + 1);
 	while (1)
 	{
-		strlcpy(newexp, exp, strlen(exp) + 1);
+		/* 
+		 * We do this to get a fresh copy of "exp"
+		 * every time thorugh the loop.
+		 * parse_inline() will mangle our string
+		 * The use of strlen(exp)+1 is intentional.
+		 */
+		strlcpy(newexp, exp, sigh);
 		ptr = parse_inline(newexp, subargs);
 		if (check_val(ptr) != whileval)
 			break;
@@ -361,7 +378,6 @@ BUILT_IN_COMMAND(foreach)
 	int	total;
 	int	i;
 	int	slen;
-	int	old_display;
 	int	list = VAR_ALIAS;
 
 	if (!subargs)
@@ -370,6 +386,12 @@ BUILT_IN_COMMAND(foreach)
 	while (args && my_isspace(*args))
 		args++;
 
+	if (!args || !*args)
+	{
+		my_error("Usage: FOREACH structure varname {block}");
+		return;
+	}
+
 	if (*args == '-')
 		args++, list = COMMAND_ALIAS;
 
@@ -404,7 +426,6 @@ BUILT_IN_COMMAND(foreach)
 	}
 
 	slen = strlen(struc);
-	old_display = window_display;
 
 	will_catch_break_exceptions++;
 	will_catch_continue_exceptions++;
@@ -451,7 +472,7 @@ BUILT_IN_COMMAND(fe)
 		*var[255],
 		*word = NULL,
 		*todo = NULL,
-		fec_buffer[2];
+		fec_buffer[16];
 	unsigned	ind, x, y;
 	int     old_display;
 	int	doing_fe = !strcmp(command, "FE");
@@ -514,7 +535,7 @@ BUILT_IN_COMMAND(fe)
 	old_display = window_display;
 
 	if (!doing_fe)
-		{ word = fec_buffer; word[1] = 0; }
+		{ word = fec_buffer; }
 
 	placeholder = templist;
 
@@ -527,7 +548,16 @@ BUILT_IN_COMMAND(fe)
 			if (doing_fe)
 				word = next_func_arg(templist, &templist);
 			else
-				word[0] = *templist ? *templist++ : 0;
+			{
+				int	codepoint;
+				codepoint = next_code_point((const unsigned char **)&templist, 0);
+				if (codepoint == -1)
+				{
+					templist++;
+					continue /* What to do? */;
+				}
+				ucs_to_utf8(codepoint, word, 16);
+			}
 
 			/* Something is really hosed if this happens */
 			if (!word)
@@ -720,13 +750,14 @@ static BUILT_IN_COMMAND(loopcmd)
  */
 BUILT_IN_COMMAND(forcmd)
 {
-	char        *working        = NULL;
-	char        *commence       = NULL;
-	char        *evaluation     = NULL;
-	char        *lameeval       = NULL;
-	char        *iteration      = NULL;
-	char        *blah           = NULL;
-	char        *commands       = NULL;
+	char *	working    = NULL;
+	char *	commence   = NULL;
+	char *	evaluation = NULL;
+	char *	lameeval   = NULL;
+	char *	iteration  = NULL;
+	char *	blah       = NULL;
+	char *	commands   = NULL;
+	size_t	sigh;
 
 	if (!subargs)
 		subargs = empty_string;
@@ -775,10 +806,16 @@ BUILT_IN_COMMAND(forcmd)
 
 	will_catch_break_exceptions++;
 	will_catch_continue_exceptions++;
-	lameeval = alloca(strlen(evaluation) + 2);
+	sigh = strlen(evaluation) + 1;
+	lameeval = alloca(sigh + 1);
 	while (1)
 	{
-		strlcpy(lameeval, evaluation, strlen(evaluation) + 1);
+		/* 
+		 * This is very intentional.
+		 * "lameeval" gets mangled every time through, so we need
+		 * to take a fresh copy from scratch each time.
+		 */
+		strlcpy(lameeval, evaluation, sigh);
 		blah = parse_inline(lameeval, subargs);
 		if (!check_val(blah))
 		{
