@@ -1,11 +1,10 @@
-/* $EPIC: ignore.c,v 1.37 2013/03/27 23:12:06 jnelson Exp $ */
 /*
  * ignore.c: handles the ingore command for irc 
  *
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  1995, 2003 EPIC Software Labs.
+ * Copyright 1995, 2003 EPIC Software Labs.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -519,7 +518,7 @@ static int	change_ignore_mask_by_desc (const char *type, Mask *do_mask, Mask *do
 
 				add_timer(0, empty_string, seconds, 1, 
 					  do_expire_ignores, NULL, NULL, 
-					  GENERAL_TIMER, -1, 0);
+					  GENERAL_TIMER, -1, 0, 0);
 			    }
 			}
 
@@ -1155,7 +1154,7 @@ char *	ignorectl (char *input)
 			seconds = time_diff(right_now, to);
 			add_timer(0, empty_string, seconds, 1,
 				do_expire_ignores, NULL, NULL, 
-				GENERAL_TIMER, -1, 0);
+				GENERAL_TIMER, -1, 0, 0);
 
 			RETURN_INT(i->refnum);
 		} else if (!my_strnicmp(listc, "REASON", len)) {
@@ -1306,7 +1305,27 @@ int	check_ignore_channel (const char *nick, const char *uh, const char *channel,
 	 * If the nickuserhost match did not say anything about the level
 	 * that we are interested in, then try a channel ignore next.
 	 */
-	else if (c_match)
+	/*
+	 * Commenting out the 'else' is an experimental change requested
+	 * by some folks who think that channel ignores should apply when
+	 * there is a nickuserhost ignore, but it does not cover the 
+	 * level we're checking for here.  As an example:
+	 *	ignore hop msgs
+	 *	ignore #epic joins
+	 * When "else" is uncommented (old behavior), you would see
+	 * joins from hop because /ignore hop suppresses /ignore #epic.
+	 * When "else" is commented out (new behavior), you will not
+	 * see joins from hop since /ignore hop doesn't cover joins.
+	 * To get the old behavior, you would have to explicitly put
+	 * an exception-ignore, a la /ignore hop msgs,!joins
+	 *
+	 * XXX This breaks backwards compatability, but it seems
+	 * like a beneficial change with the breakage only happening
+	 * in pathological situations.  Before complaining, please
+	 * demonstrate how something that was working in a sensible
+	 * way is now not behaving sensibly.
+	 */
+	/* else */ if (c_match)
 	{
 		tmp = c_match;
 		if (mask_isset(&tmp->dont, level))
