@@ -1,4 +1,3 @@
-/* $EPIC: ircsig.c,v 1.9 2008/11/28 16:28:03 jnelson Exp $ */
 /*
  * ircsig.c: has a `my_signal()' that uses sigaction().
  *
@@ -117,9 +116,9 @@ sigfunc *	init_signals (void)
 	int sig_no;
 	sigfunc *error = NULL;
 
-	memset(&signals_caught, 0, NSIG * sizeof(int));
+	memset((void *)&signals_caught, 0, NSIG * sizeof(int));
 
-	for (sig_no = 0; sig_no < NSIG; sig_no++)
+	for (sig_no = 1; sig_no < NSIG; sig_no++)
 	{
 		if ((reset_one_signal(sig_no, signal_handler)) == SIG_ERR)
 			error = SIG_ERR;
