@@ -1,11 +1,10 @@
-/* $EPIC: lastlog.c,v 1.83 2010/11/24 19:51:29 jnelson Exp $ */
 /*
  * lastlog.c: handles the lastlog features of irc. 
  *
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  1993, 2003 EPIC Software Labs.
+ * Copyright 1993, 2015 EPIC Software Labs.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,6 +44,7 @@
 #include "functions.h"
 #include "reg.h"
 #include "alias.h"
+#include "timer.h"
 
 typedef struct	lastlog_stru
 {
@@ -53,23 +53,25 @@ typedef struct	lastlog_stru
 	char	*msg;
 	struct	lastlog_stru	*older;
 	struct	lastlog_stru	*newer;
-	time_t	when;
+	time_t	created;
+	time_t	expires;
 	int	visible;
 	intmax_t refnum;
-	unsigned winref;
+	Window *window;
 	int	dead;
 }	Lastlog;
 
 static	intmax_t global_lastlog_refnum = 0;
+	double	output_expires_after = 0.0;
 
-static int	show_lastlog (Lastlog **l, int *skip, int *number, Mask *level_mask, char *match, regex_t *rex, int *max, const char *target, int mangler, unsigned winref, char **);
-static int	oldest_lastlog_for_window (Lastlog **item, unsigned winref);
-static int	newer_lastlog_entry (Lastlog **item, unsigned winref);
-static int	older_lastlog_entry (Lastlog **item, unsigned winref);
-static int	newest_lastlog_for_window (Lastlog **item, unsigned winref);
+static int	show_lastlog (Lastlog **l, int *skip, int *number, Mask *level_mask, char *match, regex_t *rex, char *nomatch, regex_t *norex, int *max, const char *target, int mangler, Window *window, int exempt, char **, int, int);
+static Lastlog *oldest_lastlog_for_window (Window *window);
+static Lastlog *newer_lastlog_entry (Lastlog *item, Window *window);
+static Lastlog *older_lastlog_entry (Lastlog *item, Window *window);
+static Lastlog *newest_lastlog_for_window (Window *window);
 static void	remove_lastlog_item (Lastlog *item);
-static void	switch_lastlog_window (Lastlog *item, unsigned newref);
-static void	move_lastlog_item (Lastlog *item, unsigned newref);
+static void	move_lastlog_item (Lastlog *item, Window *newwin);
+static void	expire_lastlog_entries (void);
 
 Lastlog *	lastlog_oldest = NULL;
 Lastlog *	lastlog_newest = NULL;
@@ -216,12 +218,22 @@ intmax_t	add_to_lastlog (Window *window, const char *line)
 	new_l->newer = NULL;
 	new_l->level = who_level;
 	new_l->msg = malloc_strdup(line);
-	new_l->winref = window->refnum;
+	new_l->window = window;
 	if (who_from)
 		new_l->target = malloc_strdup(who_from);
 	else
 		new_l->target = NULL;
-	time(&new_l->when);
+
+	time(&new_l->created);
+	if (output_expires_after != 0.0)
+	{
+		new_l->expires = time(NULL) + output_expires_after;
+		add_timer(0, empty_string, output_expires_after, 1, 
+			  do_expire_lastlog_entries, NULL, NULL, 
+			  GENERAL_TIMER, -1, 0, 0);
+	}
+	else
+		new_l->expires = 0;
 
 	/* * * */
 	if (lastlog_newest && lastlog_newest->dead)
@@ -247,20 +259,43 @@ intmax_t	add_to_lastlog (Window *window, const char *line)
 	return new_l->refnum;
 }
 
-
+/* 
+ * trim_lastlog: Remove the oldest lastlog entries for a window
+ * until the number of visible lastlog entries is no greater than 
+ * window->lastlog_max.  Note that invisible lastlog entries are
+ * only trimmed if they are older than a visible lastlog entry that
+ * was trimmed, so if the newest lastlog entries for a window are
+ * invisible then this function will never trim them.
+ */
 void 	trim_lastlog (Window *window)
 {
-	Lastlog *item = NULL;
+	Lastlog *item = oldest_lastlog_for_window(window);
 
-	while (window->lastlog_size > window->lastlog_max)
+	/* This must eventually terminate, because it will reach the end
+	 * of the linked-list of lastlog items. */
+	while (item && window->lastlog_size > window->lastlog_max)
 	{
-		item = NULL;
-		if (oldest_lastlog_for_window(&item, window->refnum))
-		{
-			if (item->visible)
-				window->lastlog_size--;
-			remove_lastlog_item(item);
-		}
+		Lastlog *next_item = newer_lastlog_entry(item, window);
+
+		remove_lastlog_item(item);
+		item = next_item;
+	}
+}
+
+/*
+ * truncate_lastlog: Removes all lastlog entries, both visible
+ * and invisible, from the given window.
+ */
+void 	truncate_lastlog (Window *window)
+{
+	Lastlog *item = oldest_lastlog_for_window(window);
+
+	while (item)
+	{
+		Lastlog *next_item = newer_lastlog_entry(item, window);
+
+		remove_lastlog_item(item);
+		item = next_item;
 	}
 }
 
@@ -282,6 +317,12 @@ void	set_lastlog_size (void *stuff)
 	{
 		window->lastlog_max = size;
 		trim_lastlog(window);
+		window_scrollback_needs_rebuild(window);
+
+		if (window->lastlog_max < window->display_lines * 2)
+			window->lastlog_max = window->display_lines * 2;
+		if (v->integer < window->display_lines * 2)
+			v->integer = window->display_lines * 2;
 	}
 }
 
@@ -293,8 +334,9 @@ void	set_lastlog_size (void *stuff)
  *	-			Do not show header and footer
  *	-reverse		Show matches newest-to-oldest (reverse order)
  *			        instead of oldest-to-newest (normal order)
- * 	-literal <pattern>	<string> is search target, not option.
+ * 	-literal <pattern>	line must wildcard match <pattern>
  *	-regex <regex>		line must match <regex>.
+ *	-ignore <pattern>	line must NOT wldcard match <pattern>
  *	-max <number>		Only show first <number> matches
  *	-skip <number>		Skip this many leading lastlog entries
  *	-number <number>	Only search this many lastlog entries
@@ -352,29 +394,36 @@ BUILT_IN_COMMAND(lastlog)
 	int		number = INT_MAX;
 	int		max = -1;
 	char *		match = NULL;
+	char *		nomatch = NULL;
 	char *		target = NULL;
 	char *		regex = NULL;
+	char *		noregex = NULL;
 	Lastlog *	start;
 	Lastlog *	end;
 	Lastlog *	l;
 	Lastlog *	lastshown;
 	regex_t 	realreg;
 	regex_t *	rex = NULL;
+	regex_t 	realnoreg;
+	regex_t *	norex = NULL;
 	int		cnt;
 	char *		arg;
 	int		header = 1;
 	Mask		save_mask;
 	int		before = -1;
 	int		after = 0;
-	int		counter = 0;
+	int		exempt_counter = 0;
 	int		show_separator = 0;
+	int		but_not_first_time = 1;
 	const char *	separator = "----";
 	char *		outfile = NULL;
 	FILE *		outfp = NULL;
 	int		mangler = 0;
 	int		lc;
 	char *		rewrite = NULL;
-	unsigned	winref = current_window->refnum;
+	Window *	window = current_window;
+	int		this_server = 0;
+	int		global = 0;
 
 	lc = message_setall(0, NULL, LEVEL_OTHER);
 	cnt = current_window->lastlog_size;
@@ -405,6 +454,22 @@ BUILT_IN_COMMAND(lastlog)
 			goto bail;
 		}
 	    }
+	    else if (!my_strnicmp(arg, "-IGNORE", len))
+	    {
+		if (!(nomatch = new_next_arg(args, &args)))
+		{
+			yell("LASTLOG -IGNORE requires an argument.");
+			goto bail;
+		}
+	    }
+	    else if (!my_strnicmp(arg, "-REGIGNORE", len))
+	    {
+		if (!(noregex = new_next_arg(args, &args)))
+		{
+			yell("LASTLOG -REGIGNORE requires an argument.");
+			goto bail;
+		}
+	    }
 	    else if (!my_strnicmp(arg, "-TARGET", len))
 	    {
 		if (!(target = new_next_arg(args, &args)))
@@ -533,8 +598,12 @@ BUILT_IN_COMMAND(lastlog)
 				"name or number", x);
 			goto bail;
 		}
-		winref = w->refnum;
+		window = w;
 	    }
+	    else if (!my_strnicmp(arg, "-THIS_SERVER", len))
+		this_server = 1;
+	    else if (!my_strnicmp(arg, "-GLOBAL", len))
+		global = 1;
 	    else if (!my_strnicmp(arg, "-", 1))
 	    {
 		int	i = str_to_level(arg+1);
@@ -595,6 +664,16 @@ BUILT_IN_COMMAND(lastlog)
 		snprintf(blah, size, "*%s*", match);
 		match = blah;
 	}
+	if (nomatch)
+	{
+		size_t	size;
+		char *	blah;
+
+		size = strlen(nomatch) + 3;
+		blah = alloca(size);
+		snprintf(blah, size, "*%s*", nomatch);
+		nomatch = blah;
+	}
 	if (regex)
 	{
 		int	options = REG_EXTENDED | REG_ICASE | REG_NOSUB;
@@ -609,12 +688,27 @@ BUILT_IN_COMMAND(lastlog)
 		}
 		rex = &realreg;
 	}
+	if (noregex)
+	{
+		int	options = REG_EXTENDED | REG_ICASE | REG_NOSUB;
+		int	errcode;
+
+		if ((errcode = regcomp(&realnoreg, noregex, options)))
+		{
+			char errmsg[1024];
+			regerror(errcode, &realreg, errmsg, 1024);
+			yell("%s", errmsg);
+			goto bail;
+		}
+		norex = &realnoreg;
+	}
 
 	if (x_debug & DEBUG_LASTLOG)
 	{
 		yell("Lastlog summary status:");
 		yell("Pattern: [%s]", match ? match : "<none>");
 		yell("Regex: [%s]", regex ? regex : "<none>");
+		yell("Ignore: [%s]", nomatch ? nomatch : "<none>");
 		yell("Target: [%s]", target ? target : "<none>");
 		yell("Header: %d", header);
 		yell("Reverse: %d", reverse);
@@ -622,10 +716,18 @@ BUILT_IN_COMMAND(lastlog)
 		yell("Number: %d", number);
 		yell("Max: %d", max);
 		yell("Mask: %s", mask_to_str(&level_mask));
+		yell("Before/After: %d/%d", before, after);
+		yell("Separator: %s", separator);
+		yell("All Windows This Server: %d", this_server);
+		yell("All Windows Globally: %d", global);
 	}
 
 	if (outfile)
 	{
+		/* 
+		 * XXX /LASTLOG -FILE is not encoding aware.
+		 * I'm not even sure if it could be.
+		 */
 		if ((outfp = fopen(outfile, "a")) == NULL)
 		{
 			say("Couldn't open output file");
@@ -642,29 +744,37 @@ BUILT_IN_COMMAND(lastlog)
 	 * take this one step at a time.  We can either go forwards or
 	 * in reverse.  Let's consider this generically:
 	 *
-	 * ALWAYS BEFORE WE "GO TO THE NEXT ITEM" if "counter" 
+	 * ALWAYS BEFORE WE "GO TO THE NEXT ITEM" if "exempt_counter" 
 	 * is greater than one we print the current item.
 	 *
-	 *	If 'counter' is greater than 0, decrease it by one.
+	 *	If 'exempt_counter' is greater than 0, decrease it by one.
 	 *	If we haven't skipped 'skip' items, go to the next item.
 	 *	If we have seen 'number' items already, then stop.
 	 *	If this item isn't of a level in 'level_mask' go to next item.
 	 *	If this item isn't matched by 'match' go to next item.
 	 *	If this item isn't regexed by 'reg' go to next item.
 	 *	-- At this point, the item "matches" and should be displayed.
-	 *	If "counter" is 0, then this is a new match on its own.
-	 *	   Go back "context" items, and set "counter" to "distance".
-	 *	Otherwise, if "counter" is not 0, then we are in the middle
-	 *	   of a previous match.  Do not go back, but set "counter"
+	 *	If "eixempt_counter" is 0, then this is a new match on its own.
+	 *	   Go back "context" items, and set "exempt_counter" to "distance".
+	 *	Otherwise, if "exempt_counter" is not 0, then we are in the middle
+	 *	   of a previous match.  Do not go back, but set "exempt_counter"
 	 *	   to "distance" (to make sure we keep outputting).
 	 */
 	if (reverse == 0)
 	{
 	    int i = 0;
 
+	    /*
+	     * Starting at the NEWEST entry, count back <number> entries.
+	     * This establishes the START POINT for our searches
+	     */
 	    for (start = end = lastlog_newest; start != lastlog_oldest; )
 	    {
-		if (start->visible && start->winref == winref)
+		if (start->visible && 
+		      (global || 
+			(this_server && 
+				start->window->server == window->server) ||
+			(start->window == window)))
 		{
 		    if (mask_isnone(&level_mask) || 
 				(mask_isset(&level_mask, start->level)))
@@ -675,48 +785,148 @@ BUILT_IN_COMMAND(lastlog)
 		start = start->older;
 	    }
 
+	    /*
+	     * Fine.  Now walk all of the lastlog entries between "start" and "end".
+	     */
 	    lastshown = NULL;
 	    for (l = start; l; (void)(l && (l = l->newer)))
 	    {
-		char *result = NULL;
+		char *result;
+		int	exempt, matching;
 
-		if (show_lastlog(&l, &skip, &number, &level_mask, 
-				match, rex, &max, target, mangler, winref,
-				&result))
+restart:
+		result = NULL;
+		exempt = 0;
+
+		/* Under any "context" situation, we unconditionally show it */
+		if (exempt_counter > 0)
 		{
-		    if (counter == 0 && before > 0)
-		    {
-			counter = 1;
+			if (x_debug & DEBUG_LASTLOG)
+				yell("This line is exempt (%d left): %s", exempt_counter, l->msg);
+			exempt = 1;
+			exempt_counter--;
+		}
+
+		/* In any case we always check if it matches (for counting purposes) */
+		matching = show_lastlog(&l, &skip, &number, &level_mask, 
+					match, rex, nomatch, norex, &max, target, 
+					mangler, window, exempt, &result, 
+					global, this_server);
+
+		/* 
+		 * Now if the present entry "matches" and we are already in a context
+		 * then we just reset the context
+		 */
+		if (matching && exempt)
+		{
+			if (x_debug & DEBUG_LASTLOG)
+				yell("I matched already in context. resetting counter: %s", l->msg);
+
+			exempt_counter = after;
+		}
+
+		/*
+		 * If the present entry "matches" and we are not already in a context,
+		 * we must see if the previous <before> lines overlapped with the
+		 * previous context (marked by ``lastshown'')
+		 */
+		else if (matching && !exempt && before > 0)
+		{
+			exempt_counter = 1;
+			show_separator = 1;
+
+			/*
+			 * So from the "match point", we walk back <before> entries.
+			 */
 			for (i = 0; i < before; i++)
 			{
+			     if (x_debug & DEBUG_LASTLOG)
+				yell("Moving back (%d of %d), now at %s", i, before, l->msg);
+
+			    /*
+			     * HOWEVER -- if we bump into something that we've already
+			     * shown (because of a previous context), we just keep 
+			     * showing from there.
+			     */
 			    if (l && l == lastshown)
-			    {
+				break;
+
+			    /*
+			     * Otherwise, we keep moving back.
+			     * Note that "counter" counts the number of lines we want
+			     * to unconditionally show!
+			     */
+			    if (l && l->older)
+				l = l->older;
+			}
+
+			if (l && l == lastshown)
+			{
+			        if (x_debug & DEBUG_LASTLOG)
+					yell("I found the previous context at %d / %s", i, l->msg);
+
 				if (l->newer)
 				    l = l->newer;
+
+				/* Don't show the separator if the contexts overlap */
 				show_separator = 0;
-				break;
-			    }
-
-			    if (l && l->older)
-			    {
-				l = l->older;
-				counter++;
-			    }
+				i--;
 			}
-		    }
-		    else if (after != -1)
-			counter = after + 1;
-		    else
-			counter = 1;
 
-		    if (show_separator)
-		    {
-			file_put_it(outfp, "%s", separator);
+			exempt_counter += i;
+			if (after != -1)
+				exempt_counter += after;
+
+			if (x_debug & DEBUG_LASTLOG)
+				yell("I matched not in context, went back %d spots, total %d exemptions: %s",
+					i, exempt_counter, l->msg);
+			goto restart;
+		}
+
+		/*
+		 * If there is "after context", then exempt that many rows.
+		 */
+		else if (matching && !exempt && after != -1)
+		{
+			exempt_counter += after;
+			if (x_debug & DEBUG_LASTLOG)
+				yell("I matched not in context, no back context, total %d exemptions: %s",
+					exempt_counter, l->msg);
+		}
+
+		/*
+		 * If there is no context at all, then exempt no rows.
+		 */
+		else if (matching)
+		{
+			exempt_counter = 0;
+			if (x_debug & DEBUG_LASTLOG)
+				yell("I matched -- no contexts! %s", l->msg);
+		}
+
+
+		/*
+		 * Now if we're showing context, we have to show the separator
+		 * between each context -- but only if this context does not 
+		 * overlap with the previous one (see above)
+		 */
+		if (show_separator)
+		{
+			if (but_not_first_time)
+				but_not_first_time = 0;
+			else
+				file_put_it(outfp, "%s", separator);
+
 			show_separator = 0;
-		    }
+		}
 
-		    if (counter)
-		    {
+		/*
+		 * Now show <counter> lines -- which will include the line
+		 * that matched, and surrounding context.
+		 */
+		if (matching || exempt)
+    		{
+			/* Honor /LOG REWRITE. */
 			if (rewrite)
 			{
 				unsigned char *n, vitals[10240];
@@ -724,8 +934,8 @@ BUILT_IN_COMMAND(lastlog)
 				snprintf(vitals, sizeof(vitals),
 					"%ld %ld %ld %ld . . . %s %s",
 						(long)l->refnum,
-						(long)l->when,
-						(long)l->winref,
+						(long)l->created,
+						(long)l->window->refnum,
 						(long)l->level,
 						l->target?l->target:".",
 						result?result:".");
@@ -737,11 +947,8 @@ BUILT_IN_COMMAND(lastlog)
 			else
 				file_put_it(outfp, "%s", result);
 
+			/* Keep track of what we have shown. */
 			lastshown = l;
-			counter--;
-			if (counter == 0 && before != -1 && separator)
-				show_separator = 1;
-		    }
 		}
 
 		new_free(&result);
@@ -749,13 +956,23 @@ BUILT_IN_COMMAND(lastlog)
 			break;
 	    }
 	}
+
+	/* 
+	 * The user specified -reverse
+	 * XXX This should be folded into the above, but there are so many
+	 * minor changes as to make that impractical. such a bummer!
+	 */
 	else
 	{
 	    int i = 0;
 
 	    for (start = end = lastlog_newest; end != lastlog_oldest; )
 	    {
-		if (end->visible && end->winref == winref)
+		if (end->visible && 
+		      (global || 
+			(this_server && 
+				end->window->server == window->server) ||
+			(end->window == window)))
 		{
 		    if (mask_isnone(&level_mask) || 
 				(mask_isset(&level_mask, end->level)))
@@ -766,48 +983,148 @@ BUILT_IN_COMMAND(lastlog)
 		end = end->older;
 	    }
 
+	    /*
+	     * Fine.  Now walk all of the lastlog entries between "start" and "end".
+	     */
 	    lastshown = NULL;
-	    for (l = start; l; (void)(l && (l = l->older)))
+	    for (l = start; l; (void)(l && (l = l->older)))	/* <<<< */
 	    {
-		char *result = NULL;
+		char *result;
+		int	exempt, matching;
 
-		if (show_lastlog(&l, &skip, &number, &level_mask, 
-				match, rex, &max, target, mangler, winref,
-				&result))
+restart2:
+		result = NULL;
+		exempt = 0;
+
+		/* Under any "context" situation, we unconditionally show it */
+		if (exempt_counter > 0)
 		{
-		    if (counter == 0 && before > 0)
-		    {
-			counter = 1;
+			if (x_debug & DEBUG_LASTLOG)
+				yell("This line is exempt (%d left): %s", exempt_counter, l->msg);
+			exempt = 1;
+			exempt_counter--;
+		}
+
+		/* In any case we always check if it matches (for counting purposes) */
+		matching = show_lastlog(&l, &skip, &number, &level_mask, 
+					match, rex, nomatch, norex, &max, target, 
+					mangler, window, exempt, &result,
+					global, this_server);
+
+		/* 
+		 * Now if the present entry "matches" and we are already in a context
+		 * then we just reset the context
+		 */
+		if (matching && exempt)
+		{
+			if (x_debug & DEBUG_LASTLOG)
+				yell("I matched already in context. resetting counter: %s", l->msg);
+
+			exempt_counter = after;
+		}
+
+		/*
+		 * If the present entry "matches" and we are not already in a context,
+		 * we must see if the previous <before> lines overlapped with the
+		 * previous context (marked by ``lastshown'')
+		 */
+		else if (matching && !exempt && before > 0)
+		{
+			exempt_counter = 1;
+			show_separator = 1;
+
+			/*
+			 * So from the "match point", we walk back <before> entries.
+			 */
 			for (i = 0; i < before; i++)
 			{
+			     if (x_debug & DEBUG_LASTLOG)
+				yell("Moving back (%d of %d), now at %s", i, before, l->msg);
+
+			    /*
+			     * HOWEVER -- if we bump into something that we've already
+			     * shown (because of a previous context), we just keep 
+			     * showing from there.
+			     */
 			    if (l && l == lastshown)
-			    {
+				break;
+
+			    /*
+			     * Otherwise, we keep moving back.
+			     * Note that "counter" counts the number of lines we want
+			     * to unconditionally show!
+			     */
+			    if (l && l->newer)		/* <<<<<< */
+				l = l->newer;		/* <<<<<< */
+			}
+
+			if (l && l == lastshown)
+			{
+			        if (x_debug & DEBUG_LASTLOG)
+					yell("I found the previous context at %d / %s", i, l->msg);
+
 				if (l->older)
 				    l = l->older;
+
+				/* Don't show the separator if the contexts overlap */
 				show_separator = 0;
-				break;
-			    }
-
-			    if (l && l->newer)
-			    {
-				l = l->newer;
-				counter++;
-			    }
+				i--;
 			}
-		    }
-		    else if (after != -1)
-			counter = after + 1;
-		    else
-			counter = 1;
 
-		    if (show_separator)
-		    {
-			file_put_it(outfp, "%s", separator);
+			exempt_counter += i;
+			if (after != -1)
+				exempt_counter += after;
+
+			if (x_debug & DEBUG_LASTLOG)
+				yell("I matched not in context, went back %d spots, total %d exemptions: %s",
+					i, exempt_counter, l->msg);
+			goto restart2;
+		}
+
+		/*
+		 * If there is "after context", then exempt that many rows.
+		 */
+		else if (matching && !exempt && after != -1)
+		{
+			exempt_counter += after;
+			if (x_debug & DEBUG_LASTLOG)
+				yell("I matched not in context, no back context, total %d exemptions: %s",
+					exempt_counter, l->msg);
+		}
+
+		/*
+		 * If there is no context at all, then exempt no rows.
+		 */
+		else if (matching)
+		{
+			exempt_counter = 0;
+			if (x_debug & DEBUG_LASTLOG)
+				yell("I matched -- no contexts! %s", l->msg);
+		}
+
+
+		/*
+		 * Now if we're showing context, we have to show the separator
+		 * between each context -- but only if this context does not 
+		 * overlap with the previous one (see above)
+		 */
+		if (show_separator)
+		{
+			if (but_not_first_time)
+				but_not_first_time = 0;
+			else
+				file_put_it(outfp, "%s", separator);
+
 			show_separator = 0;
-		    }
+		}
 
-		    if (counter)
-		    {
+		/*
+		 * Now show <counter> lines -- which will include the line
+		 * that matched, and surrounding context.
+		 */
+		if (matching || exempt)
+    		{
+			/* Honor /LOG REWRITE. */
 			if (rewrite)
 			{
 				unsigned char *n, vitals[10240];
@@ -815,8 +1132,8 @@ BUILT_IN_COMMAND(lastlog)
 				snprintf(vitals, sizeof(vitals),
 					"%ld %ld %ld %ld . . . %s %s",
 						(long)l->refnum,
-						(long)l->when,
-						(long)l->winref,
+						(long)l->created,
+						(long)l->window->refnum,
 						(long)l->level,
 						l->target?l->target:".",
 						result?result:".");
@@ -828,12 +1145,10 @@ BUILT_IN_COMMAND(lastlog)
 			else
 				file_put_it(outfp, "%s", result);
 
+			/* Keep track of what we have shown. */
 			lastshown = l;
-			counter--;
-			if (counter == 0 && before != -1 && separator)
-				show_separator = 1;
-		    }
 		}
+
 		new_free(&result);
 		if (l == end)
 			break;
@@ -846,6 +1161,8 @@ bail:
 		fclose(outfp);
 	if (rex)
 		regfree(rex);
+	if (norex)
+		regfree(norex);
 	current_window->lastlog_mask = save_mask;
 	pop_message_from(lc);
 	return;
@@ -855,19 +1172,24 @@ bail:
  * This returns 1 if the current item pointed to by 'l' is something that
  * should be displayed based on the criteron provided.
  */
-static int	show_lastlog (Lastlog **l, int *skip, int *number, Mask *level_mask, char *match, regex_t *rex, int *max, const char *target, int mangler, unsigned winref, char **result)
+static int	show_lastlog (Lastlog **l, int *skip, int *number, Mask *level_mask, char *match, regex_t *rex, char *nomatch, regex_t *norex, int *max, const char *target, int mangler, Window *window, int exempt, char **result, int global, int this_server)
 {
 	const char *str = NULL;
+	int	retval = 1;
+
 	*result = NULL;
 
-	if ((*l)->winref != winref)
+	if ( global == 1 ||
+	    (this_server == 1 && ((*l)->window->server == window->server)) ||
+	    ((*l)->window == window))
+		;	/* It's ok if this matches. */
+	else
 	{
 		if (x_debug & DEBUG_LASTLOG)
 			yell("Lastlog item belongs to another window");
 		return 0;
 	}
 
-
 	if ((*l)->visible == 0)
 	{
 		if (x_debug & DEBUG_LASTLOG)
@@ -880,7 +1202,11 @@ static int	show_lastlog (Lastlog **l, int *skip, int *number, Mask *level_mask,
 		if (x_debug & DEBUG_LASTLOG)
 			yell("Skip > 0 -- [%d]", *skip);
 		(*skip)--;	/* Have not skipped enough leading records */
-		return 0;
+
+		if (exempt)
+			retval = 0;
+		else
+			return 0;
 	}
 
 	if (!mask_isnone(level_mask) && !mask_isset(level_mask, (*l)->level))
@@ -888,7 +1214,11 @@ static int	show_lastlog (Lastlog **l, int *skip, int *number, Mask *level_mask,
 		if (x_debug & DEBUG_LASTLOG)
 			yell("Level_mask != level ([%s] [%s])",
 				mask_to_str(level_mask), level_to_str((*l)->level));
-		return 0;			/* Not of proper level */
+
+		if (exempt)
+			retval = 0;
+		else
+			return 0;			/* Not of proper level */
 	}
 
 	if (mangler)
@@ -903,37 +1233,75 @@ static int	show_lastlog (Lastlog **l, int *skip, int *number, Mask *level_mask,
 	else
 		str = (*l)->msg;
 
+
 	if (match && !wild_match(match, str))
 	{
 		if (x_debug & DEBUG_LASTLOG)
 			yell("Line [%s] not matched [%s]", str, match);
-		return 0;			/* Pattern match failed */
+
+		if (exempt)
+			retval = 0;
+		else
+			return 0;			/* Pattern match failed */
 	}
+	if (nomatch && wild_match(nomatch, str))
+	{
+		if (x_debug & DEBUG_LASTLOG)
+			yell("Line [%s] is matched [%s]", str, nomatch);
+
+		if (exempt)
+			retval = 0;
+		else
+			return 0;			/* Pattern match failed */
+	}
+
 	if (rex && regexec(rex, str, 0, NULL, 0))
 	{
 		if (x_debug & DEBUG_LASTLOG)
 			yell("Line [%s] not regexed", str);
-		return 0;			/* Regex match failed */
+
+		if (exempt)
+			retval = 0;
+		else
+			return 0;			/* Regex match failed */
+	}
+	if (norex && !regexec(norex, str, 0, NULL, 0))
+	{
+		if (x_debug & DEBUG_LASTLOG)
+			yell("Line [%s] is regexed", str);
+
+		if (exempt)
+			retval = 0;
+		else
+			return 0;			/* Regex match failed */
 	}
 	if (target && (!(*l)->target || !wild_match(target, (*l)->target)))
 	{
 		if (x_debug & DEBUG_LASTLOG)
 			yell("Target [%s] not matched [%s]", 
 					(*l)->target, target);
-		return 0;			/* Target match failed */
+
+		if (exempt)
+			retval = 0;
+		else
+			return 0;			/* Target match failed */
 	}
 	if (*max == 0)
 	{
 		if (x_debug & DEBUG_LASTLOG)
 			yell("max == 0");
 		*l = NULL;	/* Have shown maximum number of matches */
-		return 0;
+
+		if (exempt)
+			retval = 0;
+		else
+			return 0;
 	}
 	if (*max > 0)
 		(*max)--;	/* Have not yet shown max number of matches */
 
 	(*result) = malloc_strdup(str);
-	return 1;		/* Show it! */
+	return retval;		/* Show it! (or not, if exempt) */
 }
 
 /*
@@ -947,7 +1315,7 @@ void	reconstitute_scrollback (Window *window)
 
 	for (li = lastlog_oldest; li; li = li->newer)
 	{
-	    if (li->winref == window->refnum)
+	    if (li->window == window)
 		add_to_window_scrollback(window, li->msg, li->refnum);
 	}
 }
@@ -999,7 +1367,7 @@ BUILT_IN_FUNCTION(function_line, word)
 	/* Get the line from the lastlog */
 	for (start_pos = lastlog_newest; line; start_pos = start_pos->older)
 	{
-		if (start_pos->winref != win->refnum)
+		if (start_pos->window != win)
 			continue;
 		if (start_pos->visible)
 			line--;
@@ -1010,7 +1378,7 @@ BUILT_IN_FUNCTION(function_line, word)
 	else
 		start_pos = start_pos->newer;
 
-	while ((start_pos->visible == 0 || start_pos->winref != win->refnum) && 
+	while ((start_pos->visible == 0 || start_pos->window != win) && 
 				start_pos->newer)
 		start_pos = start_pos->newer;
 
@@ -1025,7 +1393,7 @@ BUILT_IN_FUNCTION(function_line, word)
 					level_to_str(start_pos->level), &clue);
 	if (do_timestamp)
 		malloc_strcat_wordlist_c(&retval, space, 
-					NUMSTR(start_pos->when), &clue);
+					NUMSTR(start_pos->created), &clue);
 
 	RETURN_MSTR(retval);
 }
@@ -1060,7 +1428,7 @@ BUILT_IN_FUNCTION(function_lastlog, word)
 
 	for (iter = lastlog_newest; iter; iter = iter->older)
 	{
-		if (iter->winref != win->refnum)
+		if (iter->window != win)
 			continue;
 		if (iter->visible == 0)
 			continue;
@@ -1154,55 +1522,52 @@ BUILT_IN_FUNCTION(function_lastlogctl, input)
 
 /************************************************************************/
 
-static int	oldest_lastlog_for_window (Lastlog **item, unsigned winref)
+static Lastlog *oldest_lastlog_for_window (Window *window)
 {
-	*item = NULL;
-	return newer_lastlog_entry(item, winref);
+	return newer_lastlog_entry(NULL, window);
 }
 
-static int	newer_lastlog_entry (Lastlog **item, unsigned winref)
+static Lastlog *newer_lastlog_entry (Lastlog *item, Window *window)
 {
 	Lastlog *i;
 
-	if (*item)
-		i = (*item)->newer;
-	else
-		i = lastlog_oldest;
+	i = item ? item->newer : lastlog_oldest;
 
-	while (i && i->winref != winref)
+	while (i && i->window != window)
 		i = i->newer;
-	*item = i;
 
-	return i ? 1 : 0;
+	return i;
 }
 
-static int	older_lastlog_entry (Lastlog **item, unsigned winref)
+static Lastlog *older_lastlog_entry (Lastlog *item, Window *window)
 {
 	Lastlog *i;
 
-	if (*item)
-		i = (*item)->older;
-	else
-		i = lastlog_newest;
+	i = item ? item->older : lastlog_newest;
 
-	while (i && i->winref != winref)
+	while (i && i->window != window)
 		i = i->older;
-	*item = i;
 
-	return i ? 1 : 0;
+	return i;
 }
 
-static int	newest_lastlog_for_window (Lastlog **item, unsigned winref)
+static Lastlog *newest_lastlog_for_window (Window *window)
 {
-	*item = NULL;
-	return older_lastlog_entry(item, winref);
+	return older_lastlog_entry(NULL, window);
+}
+
+int	recount_window_lastlog (Window *window)
+{
+	Lastlog *i;
+	int	count = 0;
+
+	for (i = lastlog_oldest; i; i = i->newer)
+		if (i->window == window && i->visible)
+			count++;
+
+	return count;
 }
 
-/*
- * Because this function does not have access to the window that this
- * lastlog item belongs to *the caller* is responsible for adjusting
- * window->lastlog_size!  I could change that in the future I guess
- */
 static void	remove_lastlog_item (Lastlog *item)
 {
 	if (item->dead)
@@ -1232,6 +1597,15 @@ static void	remove_lastlog_item (Lastlog *item)
 		item->older = NULL;
 	}
 
+	/* 
+	 * We used to do this in trim_lastlog, but it makes more sense
+	 * to do it here, doesn't it?
+	 */
+	if (item->visible)
+	{
+		item->window->lastlog_size--;
+	}
+
 	if (item->older)
 		item->older->newer = item->newer;
 	if (item->newer)
@@ -1244,80 +1618,67 @@ static void	remove_lastlog_item (Lastlog *item)
 	new_free((char **)&item);
 }
 
-static void	switch_lastlog_window (Lastlog *item, unsigned newref)
-{
-	/* Mark the old window's scrollback for reconstitution */
-	window_scrollback_needs_rebuild(item->winref);
-	item->winref = newref;
-	/* Mark the new window's scrollback for reconstitution */
-	window_scrollback_needs_rebuild(item->winref);
-}
-
 /***************************************************************************/
-static void	move_lastlog_item (Lastlog *item, unsigned newref)
+static void	move_lastlog_item (Lastlog *item, Window *newwin)
 {
-	unsigned	oldref = item->winref;
+	Window *oldwin = item->window;
 
-	item->winref = newref;
-	window_scrollback_needs_rebuild(oldref);
-	window_scrollback_needs_rebuild(newref);
+	item->window = newwin;
+	if (item->visible)
+	{
+		oldwin->lastlog_size--;
+		newwin->lastlog_size++;
+	}
+
+	window_scrollback_needs_rebuild(oldwin);
+	window_scrollback_needs_rebuild(newwin);
 }
 
-void	move_all_lastlog (unsigned oldref, unsigned newref)
+void	move_all_lastlog (Window *oldwin, Window *newwin)
 {
 	Lastlog *l;
 
 	for (l = lastlog_oldest; l; l = l->newer)
 	{
-		if (l->winref != oldref)
-			continue;
-		move_lastlog_item(l, newref);
+		if (l->window == oldwin)
+			move_lastlog_item(l, newwin);
 	}
 }
 
-void	move_lastlog_item_by_string (unsigned oldref, unsigned newref, const char *str)
+void	move_lastlog_item_by_string (Window *oldwin, Window *newwin, const char *str)
 {
 	Lastlog *l;
 
 	for (l = lastlog_oldest; l; l = l->newer)
 	{
-		if (l->winref != oldref)
-			continue;
-		if (!stristr(l->msg, str))
-			continue;
-		move_lastlog_item(l, newref);
+		if (l->window == oldwin && stristr(l->msg, str) >= 0)
+			move_lastlog_item(l, newwin);
 	}
 }
 
-void	move_lastlog_item_by_target (unsigned oldref, unsigned newref, const char *str)
+void	move_lastlog_item_by_target (Window *oldwin, Window *newwin, const char *str)
 {
 	Lastlog *l;
 
 	for (l = lastlog_oldest; l; l = l->newer)
 	{
-		if (l->winref != oldref)
-			continue;
-		if (!my_stricmp(l->target, str))
-			continue;
-		move_lastlog_item(l, newref);
+		if (l->window == oldwin && !my_stricmp(l->target, str))
+			move_lastlog_item(l, newwin);
 	}
 }
 
-void	move_lastlog_item_by_level (unsigned oldref, unsigned newref, Mask *levels)
+void	move_lastlog_item_by_level (Window *oldwin, Window *newwin, Mask *levels)
 {
 	Lastlog *l;
 
 	for (l = lastlog_oldest; l; l = l->newer)
 	{
-		if (l->winref != oldref)
-			continue;
-		if (!mask_isset(levels, l->level))
-			continue;
-		move_lastlog_item(l, newref);
+		if (l->window == oldwin && mask_isset(levels, l->level))
+			move_lastlog_item(l, newwin);
 	}
 }
 
-void	move_lastlog_item_by_regex (unsigned oldref, unsigned newref, const char *str)
+void	move_lastlog_item_by_regex (Window *oldwin, Window *newwin, const char *str)
 {
 	Lastlog *l;
 	regex_t preg;
@@ -1335,26 +1696,36 @@ void	move_lastlog_item_by_regex (unsigned oldref, unsigned newref, const char *s
 
 	for (l = lastlog_oldest; l; l = l->newer)
 	{
-		if (l->winref != oldref)
-			continue;
-		if (regexec(&preg, l->msg, 0, NULL, 0))
-			continue;
-		move_lastlog_item(l, newref);
+		if (l->window == oldwin && !regexec(&preg, l->msg, 0, NULL, 0))
+			move_lastlog_item(l, newwin);
 	}
 
 	regfree(&preg);
 }
 
-void	lastlog_swap_winrefs (unsigned oldref, unsigned newref)
+/************************************************************************/
+int	do_expire_lastlog_entries (void *ignored)
+{
+	expire_lastlog_entries();
+	return 0;
+}
+
+static void	expire_lastlog_entries (void)
 {
 	Lastlog *l;
+	time_t	nowtime;
 
+	time(&nowtime);
 	for (l = lastlog_oldest; l; l = l->newer)
 	{
-		if (l->winref == oldref)
-			l->winref = newref;
-		else if (l->winref == newref)
-			l->winref = oldref;
+		if (l->expires > 0 && l->expires <= nowtime) 
+		{
+			window_scrollback_needs_rebuild(l->window);
+			remove_lastlog_item(l);
+			if (!(l = lastlog_oldest))	/* Start over */
+				break;
+		}
 	}
 }
 
+
