@@ -1,8 +1,7 @@
-/* $EPIC: network.c,v 1.85 2009/12/29 04:20:14 jnelson Exp $ */
 /*
  * network.c -- handles stuff dealing with connecting and name resolving
  *
- * Copyright  1995, 2005 Jeremy Nelson
+ * Copyright 1995, 2007 Jeremy Nelson
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -300,16 +299,16 @@ int	inet_vhostsockaddr (int family, int port, const char *wanthost, SS *storage,
 	const char *lhn;
 
 	/*
-	 * If port == -1, then this is a client connection, so we punt
-	 * if there is no virtual host name.  But if port is NOT zero, then
-	 * the caller expects us to return a sockaddr they can bind() to, 
-	 * so we need to use LocalHostName, even if it's NULL.  If you 
+	 * If port == -1, AND "wanthost" is NULL, then this is a client connection, 
+	 * so we punt if there is no virtual host name.  But if port is NOT zero, 
+	 * then the caller expects us to return a sockaddr they can bind() to, 
+	 * so we need to use LocalIPv(4|6)HostName, even if it's NULL.  If you 
 	 * return *len == 0 for port != -1, then /dcc breaks.
 	 */
 	if ((family == AF_UNIX) || 
-            (family == AF_INET && port == -1 && LocalIPv4HostName == NULL) 
+            (family == AF_INET && port == -1 && empty(wanthost) && LocalIPv4HostName == NULL) 
 #ifdef INET6
-	 || (family == AF_INET6 && port == -1 && LocalIPv6HostName == NULL)
+	 || (family == AF_INET6 && port == -1 && empty(wanthost) && LocalIPv6HostName == NULL)
 #endif
 									   )
 	{
@@ -491,6 +490,7 @@ int	inet_ptohn (int family, const char *ip, char *retval, int size)
 {
 	SS	buffer;
 
+	memset((char *)&buffer, 0, sizeof(buffer));
 	((SA *)&buffer)->sa_family = family;
 	if (inet_strton(ip, NULL, (SA *)&buffer, AI_NUMERICHOST))
 	{
@@ -651,7 +651,7 @@ static int Connect (int fd, SA *addr)
 	set_non_blocking(fd);
 	if ((retval = connect(fd, addr, socklen(addr))))
 	{
-	    if (errno != EINPROGRESS && errno != EAGAIN)
+	    if (errno != EINPROGRESS)
 		syserr(-1, "Connect: connect(%d) failed: %s", fd, strerror(errno));
 	    else
 		retval = 0;
@@ -841,8 +841,9 @@ pid_t	async_getaddrinfo (const char *nodename, const char *servname, const AI *h
 
         if ((err = my_getaddrinfo(nodename, servname, hints, &results)))
         {
-		err = -abs(err);		/* Always a negative number */
-		write(fd, &err, sizeof(err));
+		err = -labs(err);		/* Always a negative number */
+		if (!write(fd, &err, sizeof(err))) 
+			(void) 0;
 		close(fd);
 #ifdef ASYNC_DNS
 		exit(0);
@@ -854,7 +855,8 @@ pid_t	async_getaddrinfo (const char *nodename, const char *servname, const AI *h
 	if (!results)
 	{
 		err = 0;
-		write(fd, &err, sizeof(err));
+		if (!write(fd, &err, sizeof(err))) 
+			(void) 0;
 		close(fd);
 #ifdef ASYNC_DNS
 		exit(0);
@@ -930,8 +932,10 @@ void	marshall_getaddrinfo (int fd, AI *results)
 			copy->ai_next = NULL;
 	}
 
-	write(fd, (void *)&len, sizeof(len));
-	write(fd, (void *)retval, len);
+	if (!write(fd, (void *)&len, sizeof(len))) 
+		(void) 0;
+	if (!write(fd, (void *)retval, len)) 
+		(void) 0;
 	new_free(&retval);
 }
 
