@@ -1,11 +1,10 @@
-/* $EPIC: newio.c,v 1.69 2011/04/02 07:59:42 crazyed Exp $ */
 /*
  * newio.c:  Passive, callback-driven IO handling for sockets-n-stuff.
  *
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  1997, 2007 EPIC Software Labs.
+ * Copyright 1997, 2007 EPIC Software Labs.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -149,7 +148,7 @@ static int	get_channel_by_vfd (int vfd)
 int	get_server_by_vfd (int vfd)
 {
 	if (!io_rec[vfd])
-		panic(1, "get_server_by_vfd(%d): vfd is not set up!");
+		panic(1, "get_server_by_vfd(%d): vfd is not set up!", vfd);
 	return io_rec[vfd]->server;
 }
 /* #define SRV(vfd) get_server_by_vfd(vfd) */ /* Defined in newio.h */
@@ -243,6 +242,8 @@ int	dgets_buffer (int channel, void *data, ssize_t len)
  * 2) buf    - A buffer into which to copy the data from the file descriptor
  * 3) buflen - The size of 'buf'.
  * 4) buffer - The type of buffering to perform.
+ *	-2	Fully buffered.  Don't return anything if there aren't buflen
+		bytes free.
  *	-1	No line buffering, return everything.
  *	 0	Partial line buffering.  Return a line of data if there is
  *		one; Return everything if there isn't one.
@@ -312,8 +313,8 @@ ssize_t	dgets (int vfd, char *buf, size_t buflen, int buffer)
 	 */
 	if (buffer == -2 && ioe->write_pos - ioe->read_pos < buflen)
 	{
-		yell("dgets: Wanted %d bytes, have %d bytes", 
-			ioe->write_pos - ioe->read_pos, buflen);
+		yell("dgets: Wanted %ld bytes, have %ld bytes", 
+			(long)(ioe->write_pos - ioe->read_pos), (long)buflen);
 		ioe->clean = 1;
 		kcleaned(vfd);
 		return 0;
@@ -369,8 +370,8 @@ ssize_t	dgets (int vfd, char *buf, size_t buflen, int buffer)
 	if (cnt < consumed)
 	{
 		if (x_debug & DEBUG_INBOUND) 
-			yell("VFD [%d], Truncated (did [%d], max [%d])", 
-					vfd, consumed, cnt);
+			yell("VFD [%d], Truncated (did [%ld], max [%ld])", 
+					vfd, (long)consumed, (long)cnt);
 
 		/* If the line had a newline, then put the newline in. */
 		if (buffer >= 0 && h == '\n')
@@ -400,8 +401,17 @@ ssize_t	dgets (int vfd, char *buf, size_t buflen, int buffer)
 
 /*************************************************************************/
 /*
- * do_wait: called when all of the fd's are clean, and we want to go to sleep
- * until an fd is dirty again (or a timer goes off)
+ * do_wait -- The main sleeping routine.  When all of the fd's are clean,
+ *	      go to sleep until an fd is dirty or a /TIMER goes off.
+ *
+ * Arguments:
+ *	timeout	- A value previously returned by TimerTimeout().
+ *		  'Timeout' is decremented by the time spent waiting.
+ *
+ * Return Value:
+ *	-1	Interrupted System Call (ie, EINTR caused by ^C)
+ *	 0	The timeout has expired (ie, call ExecuteTimers())
+ *	 1	An fd is dirty (ie, call do_filedesc())
  */
 int 	do_wait (Timeval *timeout)
 {
@@ -645,7 +655,7 @@ int	new_close (int vfd)
 	if (vfd >= 0 && vfd <= global_max_vfd && (ioe = io_rec[vfd]))
 	{
 		if (ioe->io_callback == ssl_read)	/* XXX */
-			shutdown_ssl(ioe->channel);
+			ssl_shutdown(ioe->channel);
 		knoread(vfd);
 		knowrite(vfd);
 
@@ -709,7 +719,7 @@ static int	unix_read (int channel, int quiet)
 	{
 		if (!quiet)
 		   syserr(CSRV(channel), "unix_read: dgets_buffer(%d, %*s) failed",
-				channel, c, buffer);
+				channel, (int)c, buffer);
 		return -1;
 	}
 
@@ -740,7 +750,7 @@ static int	unix_recv (int channel, int quiet)
 	{
 		if (!quiet)
 		   syserr(CSRV(channel), "unix_recv: dgets_buffer(%d, %*s) failed",
-				channel, c, buffer);
+				channel, (int)c, buffer);
 		return -1;
 	}
 
@@ -772,8 +782,8 @@ static int	unix_accept (int channel, int quiet)
 	if ((newfd = my_accept(channel, (SA *)&addr, &len)) < 0)
 	{
 	    if (!quiet)
-		syserr(CSRV(channel), "unix_accept: my_accept(%d) failed", 
-					": %s", channel, strerror(errno));
+		syserr(CSRV(channel), 
+			"unix_accept: my_accept(%d) failed: %s", channel, strerror(errno));
 	}
 
 	dgets_buffer(channel, &newfd, sizeof(newfd));
@@ -813,8 +823,8 @@ static int	unix_connect (int channel, int quiet)
 	getsockopt(channel, SOL_SOCKET, SO_ERROR, &sockerr, &len);
 	gso_result = errno;
 
-	dgets_buffer(channel, &sockerr, sizeof(sockerr));
 	dgets_buffer(channel, &gso_result, sizeof(gso_result));
+	dgets_buffer(channel, &sockerr, sizeof(sockerr));
 
 	/* * */
 	len = sizeof(localaddr);
