@@ -1,4 +1,3 @@
-/* $EPIC: output.c,v 1.25 2012/11/24 01:42:51 jnelson Exp $ */
 /*
  * output.c: handles a variety of tasks dealing with the output from the irc
  * program 
@@ -6,7 +5,7 @@
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  1997, 2002 EPIC Software Labs.
+ * Copyright 1997, 2002 EPIC Software Labs.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -93,6 +92,9 @@ void	redraw_all_screens (void)
 	old_os = output_screen;
 	for (s = screen_list; s; s = s->next)
 	{
+		if (!s->alive)
+			continue;
+
 		output_screen = s;
 		unflash();
 		term_clear_screen();
