@@ -1,4 +1,3 @@
-/* $EPIC: parse.c,v 1.96 2013/07/28 23:16:14 jnelson Exp $ */
 /*
  * parse.c: handles messages from the server.   Believe it or not.  I
  * certainly wouldn't if I were you. 
@@ -6,7 +5,7 @@
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  1997, 2003 EPIC Software Labs.
+ * Copyright 1997, 2003 EPIC Software Labs.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -62,7 +61,7 @@
 #define ID_CHANNEL	'!'
 
 #define space 		' '	/* Taken from rfc 1459 */
-#define	MAXPARA		15	/* Taken from rfc 1459 */
+#define	MAXPARA		20	/* RFC1459 says 15, but RusNet uses more */
 
 static	void	strip_modes (const char *, const char *, const char *);
 
@@ -326,6 +325,66 @@ static void	p_wallops (const char *from, const char *comm, const char **ArgList)
 	pop_message_from(l);
 }
 
+/*
+ * p_privmsg - Handle PRIVMSG messages from irc server
+ *
+ * Arguments:
+ *	from	- The sender of the PRIVMSG (usually a nick; servers uncommon)
+ *	comm	- The actual protocol command ("PRIVMSG")
+ *	ArgList - Arguments to the PRIVMSG:
+ *	  ArgList[0] - The receiver of the message (nick or channel)
+ *	  ArgList[1] - Payload of the message
+ *
+ * Notes:
+ *   PRIVMSGs may be sent to
+ *	1. A nick (our nick)
+ *	2. A channel (that we're on)
+ *	3. A prefix + A channel (eg "@#channel" or "+#channel")
+ *	   We treat this as #2.
+ *	4. Something else (a wall, ie, "*.iastate.edu")
+ *
+ *   PRIVMSG are sorted into several piles:
+ *	"PUBLIC" level:
+ *	1. Someone on the channel sends a message to channel	-> PUBLIC
+ *	    + The channel is current channel in any window
+ *	2. Someone on the channel sends a message to channel	-> PUBLIC_OTHER
+ *	    + The channel is NOT a current channel on any window
+ *	3. Someone not on channel sends a message to channel	-> PUBLIC_MSG
+ * 
+ *	"MSG" level:
+ *	4. A message sent to our nickname			-> MSG
+ *
+ *	"WALL" level:
+ *	5. A message sent to any other target			-> MSG_GROUP
+ *
+ *
+ *   Processing
+ *   ==========
+ *   CTCPs are delivered via PRIVMSGs, causing a rewrite of ArgList[1].
+ *	+ Most CTCPs are just removed (CTCP requests)
+ *	+ Some do in-place substitution (Encryption - ie, CTCP CAST5)
+ *	+ CTCP handling happens first, before anything below
+ *	+ CTCP handling does its own ignore, flood control, and throttling.
+ *  
+ *   If the PRIVMSG contains nothing after CTCP handling, then stop.
+ *
+ *   First, IGNOREs are checked. (CTCPs do their own ignore handling)
+ * 
+ *   If the PRIVMSG is encrypted, it will first be offered via
+ *	/ON ENCRYPTED_PRIVMSG no matter who sent it or to whom 
+ *	it was sent.
+ *
+ *   Next, flood control is checked.  
+ *
+ *   All PRIVMSGs are offered via /ON GENERAL_PRIVMSG,
+ *	no matter who sent it or to whom it was sent.  
+ *
+ *   All PRIVMSGs are offered via their respective types (see above)
+ *
+ *   Otherwise, a default message is output.
+ *
+ *   Finally, /NOTIFY is checked.
+ */
 static void	p_privmsg (const char *from, const char *comm, const char **ArgList)
 {
 	const char	*real_target, *target, *message;
@@ -695,20 +754,16 @@ static void 	p_invite (const char *from, const char *comm, const char **ArgList)
 }
 
 /* 
- * Reconnect has been put back in the "jon2 comprimise" 
+ * Received whenever we have been killed.
+ * (On old MS COMIC CHAT servers, also when someone else was killed)
  *
- * Unlimited autoreconnect for server kills
- * Limited autoreconnect for forground procis on oper kills
- *	(/set auto_reconnect ON)
- * No autoreconnect on oper kills for background procis
+ * Up through epic4, there used to be a /set auto_reconnect which was 
+ * used here, but epic5 uses server states, so p_kill is now treated
+ * as an advisory message (your script pack decides what to do once you
+ * actually get the EOF from the server).
  *
- * /on disconnect is still always hooked no matter what.
- *
- * You may configure this any way you please.. I really dont give
- * a darn what autoconnect you use... but this is a fair default.
- *
- * NOTE: If you want to change it, you *will* have to do it yourself.
- * Im quite serious.  Im not going to change this.
+ * All we do now is throw /on disconnect and/or tell you what
+ * happened and it's someone else's problem what to do with that.
  */
 static void	p_kill (const char *from, const char *comm, const char **ArgList)
 {
@@ -720,17 +775,12 @@ static void	p_kill (const char *from, const char *comm, const char **ArgList)
 	if (!(reason = ArgList[1])) { }
 
 	/* 
-	 * Bogorific Microsoft Exchange ``IRC'' server sends out a KILL
-	 * protocol message instead of a QUIT protocol message when
-	 * someone is killed on your server.  Do the obviously appropriate
-	 * thing and reroute this misdirected protocol message to 
-	 * p_quit, where it should have been sent in the first place.
-	 * Die Microsoft, Die.
+	 * Old MS Comic Chat servers (exchange) sent a KILL instead of 
+	 * a QUIT when someone was killed.  We reroute that to QUIT.
 	 */
 	if (!is_me(from_server, victim))
 	{
-		/* I don't care if this doesn't work.  */
-		p_quit(from, comm, ArgList);	/* Die Microsoft, Die */
+		p_quit(from, comm, ArgList);
 		return;
 	}
 
@@ -762,6 +812,7 @@ static void	p_kill (const char *from, const char *comm, const char **ArgList)
 						from, reason);
 	}
 
+
 	/* 
 	 * If we are a bot, and /on disconnect didnt hook, 
 	 * then we arent going anywhere.  We might as well quit.
@@ -1121,6 +1172,7 @@ static void	p_part (const char *from, const char *comm, const char **ArgList)
 		remove_channel(channel, from_server);
 	else
 		remove_from_channel(channel, from, from_server);
+
 }
 
 /*
@@ -1162,7 +1214,7 @@ static int 	p_killmsg (const char *from, const char *to, const char *cline)
 	char *poor_sap;
 	char *bastard;
 	const char *path_to_bastard;
-	char *reason;
+	const char *reason;
 	char *line;
 	int   l, retval;
 
@@ -1226,8 +1278,10 @@ static 	void 	p_snotice (const char *from, const char *to, const char *line)
 	if (!strncmp(line, "*** Notice -- ", 13))
 	{
 		if (!strncmp(line + 14, "Received KILL message for ", 26))
+		{
 			if (p_killmsg(f, to, line + 40))
 				return;
+		}
 
 		l = message_from(to, LEVEL_OPNOTE);
 		retval = do_hook(OPER_NOTICE_LIST, "%s %s", f, line + 14);
@@ -1454,6 +1508,11 @@ protocol_command rfc1459[] = {
 #define NUMBER_OF_COMMANDS (sizeof(rfc1459) / sizeof(protocol_command)) - 2;
 int 	num_protocol_cmds = -1;
 
+#define islegal(c) ((((c) >= 'A') && ((c) <= '~')) || \
+                    (((c) >= '0') && ((c) <= '9')) || \
+		     ((c) == '*') || \
+		     ((c) & 0x80))
+
 /*
  * parse_server: parses messages from the server, doing what should be done
  * with them 
@@ -1503,6 +1562,12 @@ void 	parse_server (const char *orig_line, size_t orig_line_size)
 		return;		/* Serious protocol violation -- ByeBye */
 	}
 
+	if (*from && !islegal(*from))
+	{ 
+		rfc1459_odd(from, comm, ArgList);
+		return;		
+	}
+
 	/* Some day all this needs to be replaced with an alist. */
 	if (is_number(comm))
 		numbered_command(from, comm, ArgList);
@@ -1521,3 +1586,284 @@ void 	parse_server (const char *orig_line, size_t orig_line_size)
 	FromUserHost = OldFromUserHost;
 	from_server = -1;
 }
+
+/*
+ * rfc1459_any_to_utf8	- Pre-process an RFC1459 message so it's in utf8.
+ *
+ * Arguments:
+ *	buffer - A null terminated RFC1459 message (not in utf8 already)
+ *		 Upon return, if possible, will hold the message in utf8.
+ *		 If not possible, 'extra' will hold the message.
+ *	buffsiz - How many bytes 'buffer' can hold.
+ *	extra - A pointer to NULL -- if 'buffer' is bigger than 'buffsiz'
+ *		after converting to utf8, then this will be set to a 
+ *		new_malloc()ed string.  YOU MUST new_free() THIS IF IT IS SET!
+ *
+ * This function divides an RFC1459 message into two parts
+ *	1. The "server part"
+ *	2. The "payload part"
+ * The "server part" is formatted by the server, and contains nicks and 
+ * channel names, and those will be encoded with the server's encoding.
+ * The "payload part" is encoded with whatever the sender of the message
+ * is using, which may not be the same thing as the server.
+ *
+ * The "server part" is recoded first -- which yields utf8 channels and nicks,
+ * which are then used to recode the "payload part".  The two parts are put
+ * back together into "buffer" unless the result is bigger than "buffsiz" 
+ * in which case it's put into a new_malloc()ed buffer stashed in 'extra'.
+ *
+ * XXX Ugh.  I hate that I made this so complicated just to generalize this.
+ */
+void	rfc1459_any_to_utf8 (char *buffer, size_t buffsiz, char **extra)
+{
+	char *	server_part;
+	char *	payload_part;
+	size_t	bytes_needed = 0;
+	char *	from = NULL, *to = NULL;
+	char *	command = NULL;
+	char *  endp;
+	char *	extra_server_part = NULL;
+	char *	extra_payload_part = NULL;
+	int	bytes;
+
+	if (x_debug & DEBUG_RECODE)
+		yell(">> Received %s", buffer);
+
+	/*
+	 * For the benefit of Fusion, I want to support ISO-2022-JP.
+	 * This encoding is "valid" UTF-8, but is not UTF-8.  So we need
+	 * to check it first.
+	 *
+	 * Please see http://www.sljfaq.org/afaq/encodings.html
+	 * There are other Japanese encodings that collide with UTF-8,
+	 * but I don't have any users to test, so I'm focused on what
+	 * I can actually test.
+	 */
+	if (is_iso2022_jp(buffer))
+	{
+		if (x_debug & DEBUG_RECODE)
+			yell(">> This looks like a JIS message...");
+	}
+	/* If the data is already utf8, then do nothing. */
+	else if ((bytes = invalid_utf8str(buffer)) == 0)
+	{
+		return;
+	}
+	else
+	{
+		if (x_debug & DEBUG_RECODE)
+			yell(">> There are %d invalid utf8 sequences...", bytes);
+	}
+
+	/* 
+	 * Point the "server part" at the start, and move the 
+	 * "payload part" to the argument starting with colon.
+	 */
+	server_part = buffer;
+	for (payload_part = server_part; *payload_part; payload_part++)
+	{
+		if (payload_part[0] == ' ' && payload_part[1] == ':')
+		{
+			*payload_part++ = 0;	/* Whack the space */
+			if (x_debug & DEBUG_RECODE)
+				yell(">> Found payload (%ld bytes): %s", 
+					(long)strlen(payload_part), payload_part);
+			break;
+		}
+	}
+
+	if (x_debug & DEBUG_RECODE)
+	{
+		yell(">> server part is %s, payload_part is %s", 
+			server_part, payload_part);
+	}
+
+	/* 
+	 * If "payload_part" is pointing at a nul here, there was no payload.
+	 */
+
+
+	/*
+	 * If the server part is not in utf8, then we need to convert it to
+	 * utf8 using the server's encoding to get nicks and channels in utf8.
+	 */
+	if (is_iso2022_jp(server_part) || invalid_utf8str(server_part))
+	{
+		if (x_debug & DEBUG_RECODE)
+			say(">> Need to recode server part for %d", from_server);
+
+		/* 
+		 * XXX The use of the bogus nick "zero" is just because
+		 * ``from'' can't be NULL, but we want it to use the 
+		 * server's default encoding.
+		 */
+		inbound_recode(zero, from_server, NULL, server_part, &extra_server_part);
+		if (extra_server_part)
+			server_part = extra_server_part;
+
+		if (x_debug & DEBUG_RECODE)
+			say(">> Recoded server part: %s", server_part);
+	}
+
+	/*
+	 * If the payload part exists and is not valid utf8, then we need
+	 * to figure out who sent this message, and recode it with their
+	 * encoding.  This deals with channels and nicks already in utf8.
+	 */
+	if (*payload_part && 
+	     (is_iso2022_jp(payload_part) || invalid_utf8str(payload_part))) 
+	do
+	{
+		char *	server_part_copy;
+
+		if (x_debug & DEBUG_RECODE)
+			say(">> Need to recode payload part for %d", from_server);
+
+		server_part_copy = LOCAL_COPY(server_part);
+
+		/*
+		 * Figure out who the sender is (-> "from")
+		 * Put 'endp' at the start of the command word
+		 */
+		if (*server_part_copy == ':')
+		{
+			from = server_part_copy + 1;
+			for (endp = from; *endp; endp++)
+			{
+				if (*endp == '!')
+					*endp++ = 0;
+
+				/* Not connected, so don't "fix" it */
+				if (*endp == ' ')
+				{
+					*endp++ = 0;
+					break;
+				}
+			}
+
+			/* 
+			 * So now 'from' points to the nick or server
+			 * that sent us the message, and 'endp' points
+			 * at the word after the prefix 
+			 */
+		}
+		else
+		{
+			from = NULL;
+			endp = server_part_copy;
+		}
+
+		/* Skip over the command word */
+		command = endp;
+		for (; *endp; endp++)
+		{
+			if (*endp == ' ')
+			{
+				*endp++ = 0;
+				break;
+			}
+		}
+
+		/* "endp" points at the target word (or a nul) */
+		to = endp;
+		for (; *endp; endp++)
+		{
+			if (*endp == ' ')
+			{
+				*endp++ = 0;
+				break;
+			}
+		}
+
+		if (from && !*from)
+			from = NULL;
+		if (to && !*to)
+			to = NULL;
+
+		/*
+		 * XXX UGH! BLEH! HIDEOUS!
+		 *
+		 * Some things are not recode ready.  We must detect them and
+		 * then force them to recode on their own later.
+		 * This is where the special cases are handled.
+		 */
+
+		/*
+		 * Special case #1 -- CTCP messages
+		 * Description:
+		 *    A PRIVMSG or NOTICE where the first byte of the
+		 * 	payload is \001 and the final bytes of the payload
+		 *	are \001\r\n, and there are no intervening \001s 
+		 * 	shall be treated as a well-formed CTCP message/request
+		 *	and is not subject to recoding.
+		 */
+		if (!strcmp(command, "PRIVMSG") || !strcmp(command, "NOTICE"))
+		{
+			if (payload_part[0] == ':' && payload_part[1] == '\001')
+			{
+				const char *p;
+
+				/* The second \001 must be before newline */
+				p = strchr(payload_part + 2, '\001');
+				if (p && p[1] == 0)
+					break;
+
+				/* Otherwise it's nonsense; recode it */
+			}
+		}
+
+
+		/*
+		 * Everything else isn't subject to special casing.
+		 */
+		if (x_debug & DEBUG_RECODE)
+			say(">> Recoding payload from [%s], to [%s], server [%d]", 
+				from?from:"", to?to:"", from_server);
+
+		/* UTF8-ify the payload with 'from' and 'to' */
+		inbound_recode(from, from_server, to, payload_part, &extra_payload_part);
+		if (extra_payload_part)
+			payload_part = extra_payload_part;
+
+		if (x_debug & DEBUG_RECODE)
+			say(">> Recoded payload part: %s", payload_part);
+	}
+	while (0);
+
+	/* Make copies just to get them out of 'buffer' */
+	server_part = LOCAL_COPY(server_part);
+	payload_part = LOCAL_COPY(payload_part);
+
+	if (x_debug & DEBUG_RECODE)
+		say(">> server part: %s", server_part);
+	if (x_debug & DEBUG_RECODE)
+		say(">> payload part: %s", payload_part);
+
+	/*
+	 * Now paste the two parts back together.
+	 */
+	bytes_needed = strlen(server_part);
+	if (*payload_part)
+		bytes_needed += strlen(payload_part) + 1;
+
+	if (bytes_needed > buffsiz)
+	{
+		*extra = new_malloc(bytes_needed + 2);
+		buffer = *extra;
+		buffsiz = bytes_needed + 1;
+	}
+
+	strlcpy(buffer, server_part, buffsiz);
+	if (*payload_part)
+	{
+		strlcat(buffer, " ", buffsiz);
+		strlcat(buffer, payload_part, buffsiz);
+	}
+
+	if (x_debug & DEBUG_RECODE)
+		say(">> Reconstituted UTF8 message: %s", buffer);
+
+	new_free(&extra_server_part);
+	new_free(&extra_payload_part);
+}
+
