@@ -1,8 +1,7 @@
-/* $EPIC: perl.c,v 1.23 2010/04/02 23:14:45 jnelson Exp $ */
 /*
  * perl.c -- The perl interfacing routines.
  *
- * Copyright  2001, 2006 EPIC Software Labs.
+ * Copyright 2001, 2006 EPIC Software Labs.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -205,7 +204,7 @@ char *	perlcall (char* sub, char* in, char* out, long item, char* input)
 		snprintf(retval,31,"%u",count);
 	} else {
 		SV *sv;
-		count = perl_call_pv(sub, G_EVAL|G_SCALAR);
+		perl_call_pv(sub, G_EVAL|G_SCALAR);
 		SPAGAIN ; sv=POPs ;
 		SV2STR(sv,retval);
 	}
