@@ -1,9 +1,8 @@
-/* $EPIC: reg.c,v 1.18 2005/03/19 03:55:56 jnelson Exp $ */
 /*
  * reg.c - "glob"-like wildcard pattern matching (not regexes)
  *
  * Copyright (c) 1992 Troy Rollo.
- * Copyright  1997, 2002 EPIC Software Labs.
+ * Copyright 1997, 2002 EPIC Software Labs.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -705,10 +704,9 @@ char *	pattern2regex (const char *pattern, int *weight)
 			if (pat[1] != '[')
 				goto end;
 		{
-			char *patc, *ptr, *ptr2, *arg, *placeholder;
+			char *ptr, *ptr2, *arg, *placeholder;
 			int nest = 1;
 
-			patc = LOCAL_COPY(pat);
 			placeholder = ptr = ptr2 = strstr(pattern, "\\[");
 			do
 			{
@@ -733,7 +731,7 @@ char *	pattern2regex (const char *pattern, int *weight)
 				goto end;
 
 			*ptr = 0;
-			ptr += 2;
+			/* ptr += 2; */
 			*placeholder = 0;
 			placeholder += 2;
 
