@@ -1,12 +1,11 @@
-/* $EPIC: term.c,v 1.26 2008/11/26 03:26:34 jnelson Exp $ */
 /*
  * term.c -- termios and (termcap || terminfo) handlers
  *
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  1998 J. Kean Johnston, used with permission.
- * Copyright  1995-2002 EPIC Software Labs 
+ * Copyright 1998 J. Kean Johnston, used with permission.
+ * Copyright 1995, 2008 EPIC Software Labs 
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -58,10 +57,12 @@
  */
 
 
-#ifdef _ALL_SOURCE
-#include <termios.h>
+#ifdef HAVE_TERMIOS_H
+# include <termios.h>
 #else
-#include <sys/termios.h>
+# ifdef HAVE_SYS_TERMIOS_H
+#  include <sys/termios.h>
+# endif
 #endif
 #include <sys/ioctl.h>
 
@@ -714,7 +715,7 @@ void	term_inputline_putchar (unsigned char c)
 	 * we catch the very nasty 0x9b which is the escape character with
 	 * the high bit set.  
 	 */
-	if (c < 0x20 || c == 0x9b)
+	if (c < 0x20/* || c == 0x9b*/)
 	{
 		term_standout_on();
 		putchar_x((c | 0x40) & 0x7f);
@@ -820,7 +821,7 @@ int 	term_init (void)
 		return -1;
 	}
 	else
-		fprintf(stdout, "Using terminal type [%s]\n", term);
+		printf("Using terminal type [%s]\n", term);
 
 #ifdef HAVE_TERMINFO
 	setupterm(NULL, 1, &i);
@@ -913,7 +914,7 @@ int 	term_init (void)
 	/*
 	 * Default to no colors. (ick)
 	 */
-	for (i = 0; i < 16; i++)
+	for (i = 0; i < 256; i++)
 		current_term->TI_forecolors[i] = current_term->TI_backcolors[i] = "";
 
 	if (current_term->TI_bold)
@@ -937,6 +938,11 @@ int 	term_init (void)
 		current_term->TI_sgrstrs[TERM_SGR_ALTCHAR_ON - 1] = current_term->TI_smacs;
 	if (current_term->TI_rmacs)
 		current_term->TI_sgrstrs[TERM_SGR_ALTCHAR_OFF - 1] = current_term->TI_rmacs;
+	if (current_term->TI_sitm)
+		current_term->TI_sgrstrs[TERM_SGR_ITALIC_ON - 1] = current_term->TI_sitm;
+	if (current_term->TI_ritm)
+		current_term->TI_sgrstrs[TERM_SGR_ITALIC_OFF - 1] = current_term->TI_ritm;
+
 	if (current_term->TI_normal[0])
 	{
 		current_term->TI_sgrstrs[TERM_SGR_NORMAL - 1] = current_term->TI_normal;
@@ -1059,6 +1065,20 @@ int 	term_init (void)
 		current_term->TI_backcolors[i] = malloc_strdup(cbuf);
 	}
 
+	/* I don't know any other way to set 256 colors, so ..... */
+	for (i = 16; i < 256; i++)
+	{
+		char cbuf[128];
+
+		*cbuf = 0;
+		snprintf(cbuf, sizeof cbuf, "\033[38;5;%dm", i);
+		current_term->TI_forecolors[i] = malloc_strdup(cbuf);
+
+		*cbuf = 0;
+		snprintf(cbuf, sizeof cbuf, "\033[48;5;%dm", i);
+		current_term->TI_backcolors[i] = malloc_strdup(cbuf);
+	}
+
 /* Set up the terminal discipline */
 	tty_des = 0;			/* Has to be. */
 	tcgetattr(tty_des, &oldb);
@@ -1272,6 +1292,7 @@ void	term_beep (void)
 	}
 }
 
+/* XXX Deprecated */
 void	set_meta_8bit (void *stuff)
 {
 	VARIABLE *v;
@@ -1352,6 +1373,7 @@ void	term_clrscr (void)
 	}
 }
 
+#if 0
 /*
  * Move the cursor NUM spaces to the left, non-destruvtively if we can.
  */
@@ -1370,6 +1392,7 @@ void	term_left (int num)
 		while (num--)
 			tputs_x (current_term->TI_kbs);
 }
+#endif
 
 /*
  * Move the cursor NUM spaces to the right
@@ -1387,6 +1410,7 @@ void	term_right (int num)
 			tputs_x(current_term->TI_cuf1);
 }
 
+#if 0
 /*
  * term_delete (int num)
  * Deletes NUM characters at the current position
@@ -1435,6 +1459,7 @@ void	term_repeat (unsigned char c, int rep)
 		while (rep--)
 			putchar_x (c);
 }
+#endif
 
 /*
  * Scroll the screen N lines between lines TOP and BOT.
