@@ -1,8 +1,7 @@
-/* $EPIC: who.c,v 1.65 2010/04/02 23:14:45 jnelson Exp $ */
 /*
  * who.c -- The WHO queue.  The ISON queue.  The USERHOST queue.
  *
- * Copyright  1996, 2003 EPIC Software Labs.
+ * Copyright 1996, 2003 EPIC Software Labs.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -67,7 +66,7 @@ static void	WHO_DEBUG (const char *format, ...)
 	if (who_log == NULL && (x_debug & DEBUG_WHO_QUEUE))
 	{
 		add_timer(1, who_timeref, 5, -1, who_queue_debug, 
-				NULL, NULL, GENERAL_TIMER, -1, 0);
+				NULL, NULL, GENERAL_TIMER, -1, 0, 0);
 		do_log(1, "who.log", &who_log);
 	}
 
@@ -700,8 +699,6 @@ void 	whobase (int refnum, char *args, void (*line) (int, const char *, const ch
 	}
 }
 
-static int who_whine = 0;
-
 void	whoreply (int refnum, const char *from, const char *comm, const char **ArgList)
 {
 static	char	format[40];
@@ -717,6 +714,7 @@ static	int	last_width = -1;
 	char 	*name;
 	WhoEntry *new_w = who_queue_top(refnum);
 	int	l;
+	char 	*extra = NULL;
 
 	if (!new_w)
 	{
@@ -866,7 +864,6 @@ do
 		    if (do_hook(current_numeric, "%s", buffer))
 			put_it(format, channel, nick, status, user, host, name);
 	}
-
 }
 while (new_w->piggyback && (new_w = new_w->next));
 
@@ -912,7 +909,7 @@ void	who_end (int refnum, const char *from, const char *comm, const char **ArgLi
 {
 	WhoEntry 	*new_w = who_queue_top(refnum);
 	char 		buffer[1025];
-	char		*target = malloc_strdup(ArgList[0]);
+	char		*target;
 	int		l;
 
 	if (!ArgList[0])
@@ -924,10 +921,9 @@ void	who_end (int refnum, const char *from, const char *comm, const char **ArgLi
 		return;
 	}
 
-	PasteArgs(ArgList, 0);
+	target = malloc_strdup(ArgList[0]);
 
-	if (who_whine)
-		who_whine = 0;
+	PasteArgs(ArgList, 0);
 
 	l = message_from(new_w->who_target, LEVEL_OTHER);
 	do
@@ -954,7 +950,7 @@ void	who_end (int refnum, const char *from, const char *comm, const char **ArgLi
 			*new_w->who_target = 0;
 		    }
 		}
-		else if (target && strcmp(target, new_w->who_target))
+		else if (strcmp(target, new_w->who_target))
 		{
 			WHO_DEBUG("WHOEND: Server [%d], end of who for refnum [%d]/[%s], (target was wrong: [%s])!", refnum, new_w->refnum, new_w->who_target, target);
 			*new_w->who_target = 0;
@@ -968,16 +964,20 @@ void	who_end (int refnum, const char *from, const char *comm, const char **ArgLi
 	while (new_w->piggyback && (new_w = new_w->next));
 	pop_message_from(l);
 
-	if (!*new_w->who_target)
+	if (new_w)
 	{
+	   if (!*new_w->who_target)
+	   {
 		WHO_DEBUG("Popping off for server [%d]", refnum);
 		who_queue_pop(refnum);
-	}
-	else
-	{
+	   }
+	   else
+	   {
 		WHO_DEBUG("NOT POPPING OFF FOR SERVER [%d] BECAUSE THE TOP OF THE WHO QUEUE HAS [%s] LEFT IN THE TARGET LIST", refnum, new_w->who_target);
 		yell("WHOEND: Caution -- not popping off who queue -- [%s] is left on target list", new_w->who_target);
+	   }
 	}
+	/* XXX - There should probably be an else here, to cover when new_w had a piggyback, but there was nothing to piggyback on */
 
 	new_free(&target);
 }
@@ -1000,8 +1000,6 @@ int	fake_who_end (int refnum, const char *from, const char *comm, const char *wh
 	WhoEntry 	*new_w = who_queue_top(refnum);
 	int		l;
 
-	if (who_whine)
-		who_whine = 0;
 	if (!new_w)
 		return 0;	
 
@@ -1024,7 +1022,7 @@ int	fake_who_end (int refnum, const char *from, const char *comm, const char *wh
 		if (strncmp(new_w->who_target, target, strlen(target)))
 			return 0;
 
-		who_target = target;
+		/*who_target = target; */
 	}
 
 	l = message_from(new_w->who_target, LEVEL_OTHER);
@@ -1235,7 +1233,7 @@ BUILT_IN_COMMAND(isoncmd)
 void	isonbase (int refnum, char *args, void (*line) (int, char *, char *))
 {
 	IsonEntry 	*new_i;
-	char 		*next = args, *restore = NULL;
+	char 		*next = NULL, *end, *p;
 	char		*on_cmd = NULL, *offcmd = NULL, *endcmd = NULL;
 	int		sendnext = 0;
 
@@ -1250,14 +1248,14 @@ void	isonbase (int refnum, char *args, void (*line) (int, char *, char *))
 
 		if (!my_stricmp(arg, "-d"))
 		{
-			ison_queue_list(from_server);
+			ison_queue_list(refnum);
 		}
 		if (!my_stricmp(arg, "-f"))
 		{
-			while (ison_queue_top(from_server))
-				ison_queue_pop(from_server);
-			while (ison_wait_top(from_server))
-				ison_wait_pop(from_server);
+			while (ison_queue_top(refnum))
+				ison_queue_pop(refnum);
+			while (ison_wait_top(refnum))
+				ison_wait_pop(refnum);
 		}
 		if (!my_stricmp(arg, "-s"))
 		{
@@ -1267,21 +1265,29 @@ void	isonbase (int refnum, char *args, void (*line) (int, char *, char *))
 		{
 			sendnext++;
 		}
-		if (!my_stricmp(arg, "-e") && get_server(refnum)->ison_wait)
+		if (!my_stricmp(arg, "-e"))
 		{
-			return;
+			if (get_server(refnum))
+				if (get_server(refnum)->ison_wait)
+					return;
 		}
 		if (!my_stricmp(arg, "-len"))
 		{
 			if ((stuff = next_arg(args, &args)))
+			{
+			     if (get_server(refnum))
 				get_server(refnum)->ison_len = MAX(100, atol(stuff));
+			}
 			else
 				say("Need numeric argument for -LEN argument.");
 		}
 		if (!my_stricmp(arg, "-max"))
 		{
 			if ((stuff = next_arg(args, &args)))
+			{
+			    if (get_server(refnum))
 				get_server(refnum)->ison_max = atol(stuff);
+			}
 			else
 				say("Need numeric argument for -MAX argument.");
 		}
@@ -1309,33 +1315,115 @@ void	isonbase (int refnum, char *args, void (*line) (int, char *, char *))
 	}
 
 	ison_queue_send(refnum);
-	if (!args || !*args)
+	if (!args)
 		return;
 
-	next = args;
-	while ((args = next))
-	{
-		new_i = get_new_ison_entry(refnum, sendnext);
-		new_i->line = line;
-		if ((int)strlen(args) > get_server(refnum)->ison_len)
-		{
-			next = args + get_server(refnum)->ison_len;
-			while (!isspace(*next))
-				next--;
-			restore = next;
-			*next++ = 0;
-		}
-		else
-			next = NULL;
 
-		malloc_strcpy(&new_i->ison_asked, args);
-		malloc_strcpy(&new_i->oncmd, on_cmd);
-		malloc_strcpy(&new_i->offcmd, offcmd);
-		if (!next)
-			malloc_strcpy(&new_i->endcmd, endcmd);
+	/* So now we need to go through 'args', collecting the nicks into groups
+	 * of at most ison_len and sending those off.  Two things conspire
+	 * to make this harder than it sounds, however: an individual nick might
+	 * be longer than ison_len on its own, in which case we have to ignore it;
+	 * and if an endcmd was supplied then we must only add it to the _last_
+	 * queued ison command.  We don't know if a given queued ison command
+	 * is going to be the last one until we look ahead and hit the end of
+	 * the arguments. 
+	 */
+	new_i = NULL;
+	end = NULL;
+	p = args;
+	
+	while (*p)
+	{
+		/* Loop pre-conditions:
+		 *
+		 * p is pointing at start of next argument to examine, or at the
+		 * terminating null.
+		 *
+		 * end is pointing at the end of the current list of nicks to send,
+		 * or NULL if that list is empty. 
+		 *
+		 * if end != NULL, next is pointing at start of the current list of
+		 * nicks to send. 
+		 *
+		 * if new_i != NULL && end != NULL, new_i is pointing at the ison 
+		 * entry for the current list of nicks to send (new_i->ison_asked is
+		 * is NOT set yet).
+		 *
+		 * if new_i != NULL && end == NULL then new_i is pointing at the ison
+		 * entry for the previous list of nicks to send (new_i->ison_asked is
+		 * set). 
+		 */
+		if (!end)
+			next = p;
+		while (*p && !isspace((unsigned char)*p))
+		    p++;
+
+		/* You know, just in case something happened... */
+		if (get_server(refnum))
+		{
+		    if (p - next < get_server(refnum)->ison_len)
+		    {
+			/* can add this nick to the current list. */
+			if (!end)
+			{
+				/* This is the first nick of a new ison list, so now we know that
+				 * the previous list, if there was one, wasn't the last, so we can
+				 * send it off without an endcmd. */
+				if (new_i)
+					ison_queue_send(refnum);
+	
+				if (!(new_i = get_new_ison_entry(refnum, sendnext)))
+				{
+					privileged_yell("isonbase: Could not create a new ison entry for %d [%d]", refnum, sendnext);
+					continue;
+				}
+				new_i->line = line;
+				malloc_strcpy(&new_i->oncmd, on_cmd);
+				malloc_strcpy(&new_i->offcmd, offcmd);
+			}
+				
+			end = p;
+		    }
+		    else
+		    {
+			/* This nick is too long to add to the current list. */
+			if (end)
+			{
+				/* ..but there are some preceding words that we can send */
+				char restore = *end;
+	
+				*end = 0;
+				malloc_strcpy(&new_i->ison_asked, next);
+				*end = restore;
+				/* Note that we do NOT send the ison queue yet, because we
+				 * don't yet know if we should set endcmd or not. */
+	
+				p = end;	
+				end = NULL;
+			}
+		    }
+		}
+		
+		/* Advance to next nick */
+		while (isspace((unsigned char)*p))
+			p++;
+	}
+		
+	if (new_i)
+	{
+		/* Send the last list - this one has endcmd set */
+		if (end)
+		{
+			/* ison_asked hasn't been set yet */
+			char restore = *end;
+
+			*end = 0;
+			malloc_strcpy(&new_i->ison_asked, next);
+			*end = restore;
+		}
+
+		malloc_strcpy(&new_i->endcmd, endcmd);
 		ison_queue_send(refnum);
-		if (restore)
-			*restore = ' ';
 	}
 }
 
@@ -1486,6 +1574,7 @@ static void userhost_entry_pop (UserhostEntry **entry)
 {
 	UserhostEntry *save = (*entry)->next;
 
+	/* XXX But what if *entry is null? */
 	new_free(&(*entry)->userhost_asked);
 	new_free(&(*entry)->text);
 	new_free((char **)entry);
@@ -1764,6 +1853,7 @@ void	userhost_returned (int refnum, const char *from, const char *comm, const ch
 				yell("Can't parse useless USERHOST reply [%s]", 
 						ArgList[0]);
 				userhost_queue_pop(refnum);
+				return;
 			}
 
 			if (user[-1] == '*')
